// Entry point to the WebSocket interface
var gateway = `ws://${window.location.hostname}/ws`;
var websocket;

// ----------------------------------------------------------------------------
// Initialization
// ----------------------------------------------------------------------------
window.addEventListener('load', onLoad);

function onLoad(event) {
  initWebSocket();
  initInputText();
  clearTextArea();
}

// ----------------------------------------------------------------------------
// WebSocket handling
// ----------------------------------------------------------------------------
function initWebSocket() {
  console.log('Trying to open a WebSocket connection...');
  websocket = new WebSocket(gateway);
  websocket.onopen = onOpen;
  websocket.onclose = onClose;
  // React to server's 'textAll()' function
  websocket.onmessage = onMessage;
}

function onOpen(event) {
  console.log('Connection opened');
}

function onClose(event) {
  console.log('Connection closed');
  setTimeout(initWebSocket, 2000);
}

function onMessage(event) {
  sessionStorage.setItem('textArea', sessionStorage.getItem('textArea') + event.data);
  var myResponse = document.getElementById('response');
  myResponse.value = sessionStorage.getItem('textArea');
  myResponse.scrollTop = myResponse.scrollHeight;
}

// ----------------------------------------------------------------------------
// Command page input text handling
// ----------------------------------------------------------------------------
function initInputText() {
  document.getElementById('command').addEventListener('keydown', onKeyDown);
}

// ----------------------------------------------------------------------------
// Command page button handling
// ----------------------------------------------------------------------------
clearTextArea();
function clearTextArea() {
  document.getElementById('response').value = '';
  sessionStorage.setItem('textArea', '');
  document.getElementById('command').focus();
}

function sendCommand() {
  var cmd = document.getElementById('command');
  websocket.send(cmd.value);
  cmd.value = '';
  cmd.focus();
}

function onKeyDown(event) {
  if (event.keyCode == 13) {
    sendCommand();
  }
}

function sendSpecCommand(val) {
  websocket.send(val);
  document.getElementById('command').focus();
}

function powerOff() {
  if (confirm('Do you really want to terminate and power off Waterbox?')) {
    window.location.href = '/terminate';
  }
}
