/*
 * Commands.h
 *
 * Created on: 20 Feb 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef COMMANDS_H_
#define COMMANDS_H_

#include <Arduino.h>


class Commands {
  public:
    // Find the index of the array of the slot which contains the
    // parameter string.
    // Answer -1, if string not found
    static int find(String string);

    // Execute the i-th function.
    static void executeFunction(int i);

    // Public auxiliary functions
    static void printVersionInstitution();
    static void printConnectedTo(String str);
    static void setSerial();
    static void setSerial1();
    static void printCurrentTimeLn();
    static void printMonitorTemp();
    static void printMonitorConductivity();
    static void printMonitorTurbidity();

    // System power off flag
    static boolean toPowerOff;

  private:
    // Get the i-th command (maybe with parameters) from the menu table
    static String getMenuTable(int i);

    // Get the i-th command text from the text table
    static String getTextTable(int i);

    // Private auxiliary functions
    static void printSpaces(int num);
    static void printDots(int num);
    static void printHelpLine(int i, int tab);
    static void printMessage(String str);
    static void connectSerial(String str, Stream* stream);
    static void printCurrentTime();
    static void collectLogfileHeaders();
    static void temp();
    static void conductivity();
    static void turbidity();
    static void printLoggerRunning();
    static void printLcdInterval();
    // --- HELP --------------------------------------------------------------------------------------
    static void printHelpPT();
    static void printHelpSensorsPT();
    static void printHelpTimeDatePT();
    static void printHelpLogfilePT();
    static void printHelpLoggerPT();
    static void printHelpMonitorPT();
    static void printHelpLcdPT();
    static void printHelpMiscPT();
    static void printHelpDevelPT();
    // --- SENSORS -----------------------------------------------------------------------------------
    static void printTempPT();
    static void printTempCalibValuesPT();
    static void setTempCalibLowValuesPT();
    static void setTempCalibHighValuesPT();
    static void printConductivityPT();
    static void printConductivityCalibValuesPT();
    static void setConductivityCalibValuesPT();
    static void printTurbidityPT();
    static void printTurbidityCalibValuesPT();
    // --- TIME AND DATE -----------------------------------------------------------------------------
    static void printDatePT();
    static void printTimePT();
    static void setDatePT();
    static void setTimePT();
    // --- LOGFILE -----------------------------------------------------------------------------------
    static void printLogfileNamePT();
    static void setLogfileNamePT();
    static void printPartialLogfilePT();
    // --- LOGGER ------------------------------------------------------------------------------------
    static void setLoggerStartTimeTodayPT();
    static void setLoggerStartDatePT();
    static void setLoggerStopTimeTodayPT();
    static void setLoggerStopDatePT();
    static void setLoggerIntervalPT();
    static void setLoggerIntervalDaysPT();
    static void resetLoggerPT();
    static void printLoggerSettingsPT();
    static void logTempOnPT();
    static void logTempOffPT();
    static void logConductivityOnPT();
    static void logConductivityOffPT();
    static void logTurbidityOnPT();
    static void logTurbidityOffPT();
    static void loggerRunPT();
    static void loggerHaltPT();
    // --- MONITOR -----------------------------------------------------------------------------------
    static void setMonitorStartTimeTodayPT();
    static void setMonitorStartDatePT();
    static void setMonitorStopTimeTodayPT();
    static void setMonitorStopDatePT();
    static void setMonitorIntervalPT();
    static void setMonitorIntervalDaysPT();
    static void resetMonitorPT();
    static void printMonitorSettingsPT();
    static void monitorTempOnPT();
    static void monitorTempOffPT();
    static void monitorConductivityOnPT();
    static void monitorConductivityOffPT();
    static void monitorTurbidityOnPT();
    static void monitorTurbidityOffPT();
    static void monitorRunPT();
    static void monitorHaltPT();
    // --- LCD DISPLAY AND BUTTON MODE ---
    static void storeLcdLogfileNamePT();
    static void setLcdLoggerIntervalPT();
    static void setBacklightTimeoutPT();
    static void printLcdSettingsPT();
    // --- MISCELLANEOUS -----------------------------------------------------------------------------
    static void printVersionInstitutionPT();
    static void printSdCardInfoPT();
    static void setEchoOnPT();
    static void setEchoOffPT();
    static void setSerialPT();
    static void wifiPT();
    static void setNetPT();
    static void setApPT();
    static void setNetCredentialsPT();
    static void setApCredentialsPT();
    static void printLicensePT();
    static void poweroffPT();
    // --- DEVELOPMENT -------------------------------------------------------------------------------
    static void setMonitorSettingsPT();
    static void setLoggerSettingsPT();
    static void testrunMonitorPT();
    static void testrunLoggerPT();
    static void copyMonitorDatesToLoggerPT();
    static void fakesetTempCalibLowValuesPT();
    static void fakesetTempCalibHighValuesPT();
    static void fakesetConductivityCalibValuesPT();
    static void printPrefsPT();
    static void resetPrefsPT();
    static void enableSerial1PT();
    static void disableSerial1PT();
    static void setSerial1PT();
    static void startLcdPT();
    static void stopLcdPT();
    static void connectWifiPT();
    static void disconnectWifiPT();
    static void directoryPT();
    static void test0PT();
    static void test1PT();
    static void test2PT();
    static void test3PT();
    static void test4PT();
    static void test5PT();
    static void test6PT();
    static void test7PT();
    static void test8PT();
    static void test9PT();

    // Buffer
    static char buf[];
    // Array with commands
    static const char* const MENU_TABLE[];
    // Array with command text
    static const char* const TEXT_TABLE[];
    // Array with function pointers
    static void (* const FP_TABLE[])();

    // The menu strings stored in and retrieved from flash memory instead of into RAM.
    // --- HELP ---------------------------------------------------------------------
    static const char MENU_HELP_00[];   static const char TEXT_HELP_00[];
    static const char MENU_HELP_01[];   static const char TEXT_HELP_01[];
    static const char MENU_HELP_02[];   static const char TEXT_HELP_02[];
    static const char MENU_HELP_03[];   static const char TEXT_HELP_03[];
    static const char MENU_HELP_04[];   static const char TEXT_HELP_04[];
    static const char MENU_HELP_05[];   static const char TEXT_HELP_05[];
    static const char MENU_HELP_06[];   static const char TEXT_HELP_06[];
    static const char MENU_HELP_07[];   static const char TEXT_HELP_07[];
    static const char MENU_HELP_08[];   static const char TEXT_HELP_08[];
    // --- SENSORS ------------------------------------------------------------------
    static const char MENU_SENSORS_00[];   static const char TEXT_SENSORS_00[];
    static const char MENU_SENSORS_01[];   static const char TEXT_SENSORS_01[];
    static const char MENU_SENSORS_02[];   static const char TEXT_SENSORS_02[];
    static const char MENU_SENSORS_03[];   static const char TEXT_SENSORS_03[];
    static const char MENU_SENSORS_04[];   static const char TEXT_SENSORS_04[];
    static const char MENU_SENSORS_05[];   static const char TEXT_SENSORS_05[];
    static const char MENU_SENSORS_06[];   static const char TEXT_SENSORS_06[];
    static const char MENU_SENSORS_07[];   static const char TEXT_SENSORS_07[];
    static const char MENU_SENSORS_08[];   static const char TEXT_SENSORS_08[];
    // --- DATE/TIME ----------------------------------------------------------------
    static const char MENU_DATETIME_00[];   static const char TEXT_DATETIME_00[];
    static const char MENU_DATETIME_01[];   static const char TEXT_DATETIME_01[];
    static const char MENU_DATETIME_02[];   static const char TEXT_DATETIME_02[];
    static const char MENU_DATETIME_03[];   static const char TEXT_DATETIME_03[];
    // --- LOGFILE ------------------------------------------------------------------
    static const char MENU_LOGFILE_00[];   static const char TEXT_LOGFILE_00[];
    static const char MENU_LOGFILE_01[];   static const char TEXT_LOGFILE_01[];
    static const char MENU_LOGFILE_02[];   static const char TEXT_LOGFILE_02[];
    // --- LOGGER -------------------------------------------------------------------
    static const char MENU_LOGGER_00[];   static const char TEXT_LOGGER_00[];
    static const char MENU_LOGGER_01[];   static const char TEXT_LOGGER_01[];
    static const char MENU_LOGGER_02[];   static const char TEXT_LOGGER_02[];
    static const char MENU_LOGGER_03[];   static const char TEXT_LOGGER_03[];
    static const char MENU_LOGGER_04[];   static const char TEXT_LOGGER_04[];
    static const char MENU_LOGGER_05[];   static const char TEXT_LOGGER_05[];
    static const char MENU_LOGGER_06[];   static const char TEXT_LOGGER_06[];
    static const char MENU_LOGGER_07[];   static const char TEXT_LOGGER_07[];
    static const char MENU_LOGGER_08[];   static const char TEXT_LOGGER_08[];
    static const char MENU_LOGGER_09[];   static const char TEXT_LOGGER_09[];
    static const char MENU_LOGGER_10[];   static const char TEXT_LOGGER_10[];
    static const char MENU_LOGGER_11[];   static const char TEXT_LOGGER_11[];
    static const char MENU_LOGGER_12[];   static const char TEXT_LOGGER_12[];
    static const char MENU_LOGGER_13[];   static const char TEXT_LOGGER_13[];
    static const char MENU_LOGGER_14[];   static const char TEXT_LOGGER_14[];
    static const char MENU_LOGGER_15[];   static const char TEXT_LOGGER_15[];
    static const char MENU_LOGGER_16[];   static const char TEXT_LOGGER_16[];
    static const char MENU_LOGGER_17[];   static const char TEXT_LOGGER_17[];
    // --- MONITOR ------------------------------------------------------------------
    static const char MENU_MONITOR_00[];  static const char TEXT_MONITOR_00[];
    static const char MENU_MONITOR_01[];  static const char TEXT_MONITOR_01[];
    static const char MENU_MONITOR_02[];  static const char TEXT_MONITOR_02[];
    static const char MENU_MONITOR_03[];  static const char TEXT_MONITOR_03[];
    static const char MENU_MONITOR_04[];  static const char TEXT_MONITOR_04[];
    static const char MENU_MONITOR_05[];  static const char TEXT_MONITOR_05[];
    static const char MENU_MONITOR_06[];  static const char TEXT_MONITOR_06[];
    static const char MENU_MONITOR_07[];  static const char TEXT_MONITOR_07[];
    static const char MENU_MONITOR_08[];  static const char TEXT_MONITOR_08[];
    static const char MENU_MONITOR_09[];  static const char TEXT_MONITOR_09[];
    static const char MENU_MONITOR_10[];  static const char TEXT_MONITOR_10[];
    static const char MENU_MONITOR_11[];  static const char TEXT_MONITOR_11[];
    static const char MENU_MONITOR_12[];  static const char TEXT_MONITOR_12[];
    static const char MENU_MONITOR_13[];  static const char TEXT_MONITOR_13[];
    static const char MENU_MONITOR_14[];  static const char TEXT_MONITOR_14[];
    static const char MENU_MONITOR_15[];  static const char TEXT_MONITOR_15[];
    static const char MENU_MONITOR_16[];  static const char TEXT_MONITOR_16[];
    static const char MENU_MONITOR_17[];  static const char TEXT_MONITOR_17[];
    // --- LCD DISPLAY --------------------------------------------------------------
    static const char MENU_LCD_00[];   static const char TEXT_LCD_00[];
    static const char MENU_LCD_01[];   static const char TEXT_LCD_01[];
    static const char MENU_LCD_02[];   static const char TEXT_LCD_02[];
    static const char MENU_LCD_03[];   static const char TEXT_LCD_03[];
    // --- MISCELLANEOUS ------------------------------------------------------------
    static const char MENU_MISC_00[];   static const char TEXT_MISC_00[];
    static const char MENU_MISC_01[];   static const char TEXT_MISC_01[];
    static const char MENU_MISC_02[];   static const char TEXT_MISC_02[];
    static const char MENU_MISC_03[];   static const char TEXT_MISC_03[];
    static const char MENU_MISC_04[];   static const char TEXT_MISC_04[];
    static const char MENU_MISC_05[];   static const char TEXT_MISC_05[];
    static const char MENU_MISC_06[];   static const char TEXT_MISC_06[];
    static const char MENU_MISC_07[];   static const char TEXT_MISC_07[];
    static const char MENU_MISC_08[];   static const char TEXT_MISC_08[];
    static const char MENU_MISC_09[];   static const char TEXT_MISC_09[];
    static const char MENU_MISC_10[];   static const char TEXT_MISC_10[];
    static const char MENU_MISC_11[];   static const char TEXT_MISC_11[];
    // --- DEVEL --------------------------------------------------------------------
    static const char MENU_DEVEL_00[];   static const char TEXT_DEVEL_00[];
    static const char MENU_DEVEL_01[];   static const char TEXT_DEVEL_01[];
    static const char MENU_DEVEL_02[];   static const char TEXT_DEVEL_02[];
    static const char MENU_DEVEL_03[];   static const char TEXT_DEVEL_03[];
    static const char MENU_DEVEL_04[];   static const char TEXT_DEVEL_04[];
    static const char MENU_DEVEL_05[];   static const char TEXT_DEVEL_05[];
    static const char MENU_DEVEL_06[];   static const char TEXT_DEVEL_06[];
    static const char MENU_DEVEL_07[];   static const char TEXT_DEVEL_07[];
    static const char MENU_DEVEL_08[];   static const char TEXT_DEVEL_08[];
    static const char MENU_DEVEL_09[];   static const char TEXT_DEVEL_09[];
    static const char MENU_DEVEL_10[];   static const char TEXT_DEVEL_10[];
    static const char MENU_DEVEL_11[];   static const char TEXT_DEVEL_11[];
    static const char MENU_DEVEL_12[];   static const char TEXT_DEVEL_12[];
    static const char MENU_DEVEL_13[];   static const char TEXT_DEVEL_13[];
    static const char MENU_DEVEL_14[];   static const char TEXT_DEVEL_14[];
    static const char MENU_DEVEL_15[];   static const char TEXT_DEVEL_15[];
    static const char MENU_DEVEL_16[];   static const char TEXT_DEVEL_16[];
    static const char MENU_DEVEL_17[];   static const char TEXT_DEVEL_17[];
    static const char MENU_DEVEL_18[];   static const char TEXT_DEVEL_18[];
    static const char MENU_DEVEL_19[];   static const char TEXT_DEVEL_19[];
    static const char MENU_DEVEL_20[];   static const char TEXT_DEVEL_20[];
    static const char MENU_DEVEL_21[];   static const char TEXT_DEVEL_21[];
    static const char MENU_DEVEL_22[];   static const char TEXT_DEVEL_22[];
    static const char MENU_DEVEL_23[];   static const char TEXT_DEVEL_23[];
    static const char MENU_DEVEL_24[];   static const char TEXT_DEVEL_24[];
    static const char MENU_DEVEL_25[];   static const char TEXT_DEVEL_25[];
    static const char MENU_DEVEL_26[];   static const char TEXT_DEVEL_26[];
    static const char MENU_DEVEL_27[];   static const char TEXT_DEVEL_27[];
};

#endif /* COMMANDS_H_ */
