/*
 * InputStreamReader.h
 *
 * Created on: 9 Apr 2021
 * Author: ruedi.mueller@fhnw.ch
 *
 * This class implements an incremental (i.e. non-blocking) input
 * stream reader which is to be used in the main loop.
 * readInput() reads stream input from  Stream* Config::instream
 * to complete a command string. Completion is recognised when the
 * terminator char (to be defined via 'InputStreamReader::begin') is
 * detected. isComplete() answers this question.
 * Upon completion the input is available via getInput().
 */

#ifndef INPUTSTREAMREADER_H_
#define INPUTSTREAMREADER_H_

#include <Arduino.h>


class InputStreamReader {
  public:
    // Define terminator which signals completion of input
    static void begin(char terminator);

    // Read next char to complete input
    static void readInput();

    // Answer whether input is complete
    static bool isComplete();

    // Get the input.
    // Only use after isComplete() answered with true!
    static String getInput();

  private:
    // The input terminator
    static char terminator;
    
    // The last read char
    static char c;

    // The input to complete
    static String input;
    
    // Flag to indicate whether input is complete
    static bool complete;
};

#endif /* INPUTSTREAMREADER_H_ */
