/*
 * FSM.h
 *
 * Created on: 8 May 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef LCDFSM_H_
#define LCDFSM_H_

#include "LcdState.h"


class LcdFSM {
  public:
    // Constant to be used if no condition has to be met
    // Example: FSM::nextState(FSM::NO_CONDITION)
    static const int NO_CONDITION;

    // Init class
    static void begin(LcdState* currentState);

    // Proceed to the next state, if the condition of the condition function
    // is met, else remain in the old state.
    // If a new state becomes the current state the corresponding transition
    // action (function) is executed.
    static void nextState();

    // Print the current state of the FSM
    static void printCurrentState();
    
    // Set the current condition function
    static void setConditionFunction(int (*conditionFunction)());

    static LcdState* getCurrentState();

    // Millis helper functions
    static void printMillis();
    static void printElapsedMillis();
    static void resetMillis();
    
    // State transition condition function
    // Set FSM to be time sensitive.
    static int millisCondFunction();

  private:
    // The current state of the FSM
    static LcdState* currentState;

    static int (*conditionFunction)();

    static unsigned long startMillis;
};

#endif /* LCDFSM_H_ */
