/*
 * LcdState.h
 *
 * Created on: 8 May 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef LCDSTATE_H_
#define LCDSTATE_H_

#include <Arduino.h>

// #define NAMELESS_STATE


class LcdState {
  public:
    // Name of the state
    LcdState(String name);

#ifndef NAMELESS_STATE
    // Get the state's name
    String getName();

    // Print connected states with their names
    void printConnectedStates();
#endif

    // Access the array with all the conditions
    int* getConditions();

    // Get the connected state by its index
    LcdState* getConnectedStates(int i);

    // Perform the transition action of the i-th state
    void performAction(int i);
    
    void setArrays(LcdState** connectedStates, int* conditions, void (**actions)(), int len);

    unsigned int arrSize();

  private:
#ifndef NAMELESS_STATE
    // The name of the state
    String name;
#endif

    // Pointer to connected states pointers
    LcdState** connectedStatesPtr;  // Pointer to array of pointers to State pointers

    // Pointer transition condition values
    int* conditionsPtr;  // Pointer to array of int

    // Pointer transition actions
    void (** actionsPtr)();  // Pointer to array of pointers to functions returning void

    unsigned int length;
};

#endif /* LCDSTATE_H_ */
