/*
 * LcdText.h
 *
 * Created on: 23 May 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef LCDTEXT_H_
#define LCDTEXT_H_

#include <Arduino.h>
#include <LiquidCrystal_I2C.h>


class LcdText {
  public:
    // Reset 'once' flags
    static void resetOnces();

    static void powerOnScreen();
    static void powerOffText();

    // Provide transition functions to be performed
    // if a state change succeeds (i.e. due to timing)
    static void pSplashScreen();
    static void pLgReadyScreen1();
    static void pLgReadyScreen2();
    static void pLgReadyScreen3();

    // Enter power off mode
    static void pLgPwrOffScreen1();
    static void pLgPwrOffScreen2();
    static void pLgPwrOffScreen3();

    // Enter WiFi mode
    static void pLgWifiScreen();
    static void printRotatePattern();
    static void printIP(String ip);

    // Create file
    static void pLgRunScreenWriteFile();
    // Add to current file; idle till 2000 msecs have passed as
    // writing takes time
    static void pLgRunScreenWriteValues();
    
    static void pLgRunScreen1();
    static void pLgRunScreen2();
    static void pLgRunScreen3();
    static void pLgPause1();
    static void pLgPause2();

    static void pLgRunBlinkOff();

    static void p2Countdown();
    static void p1Countdown();
    static void p0Countdown();

    static void printDateTime();

    static void printSdNotPresent();
    // Print that file count cannot outnumber 99
    static void printFileCount99();

    static void printSerialActive();
    static void printSerial1Active();

    static void wifiSymbol();
    static void wifiNoSymbol();
    static void powerOffSymbol();

    static void error(String str);

  private:
    // Pointer to lcd object
    static LiquidCrystal_I2C* wbLcd;

    // Arrow symbol for LCD display
    static byte arrow[8];

    // State var for rotatePattern()
    static int patternPos;

    // Blinking: State flags used to print arrow once only
    static bool erased;
    static bool written;
    // Countdown text: State flags used to print countdown text once only
    static bool once2;
    static bool once1;
    static bool once0;

    static void textBorder();
    static void printSensorValues();
    static void printClearLastLine();

    // Print values on second and third lcd line
    static void printValuesStrings();
};

#endif /* LCDTEXT_H_ */
