/*
 * Monitor.h
 *
 * Created on: 18 Feb 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef MONITOR_H_
#define MONITOR_H_

#include "Timer.h"


class Monitor {
  public:
	  // The monitor's timer
	  static Timer timer;

    // Flag whether temperature is to be monitored or not
    static void setTempOn(bool flag);
    // Answer true if temp monitoring is on
    static bool isTempOn();

    // Flag whether conductivity is to be monitored or not
    static void setConductivityOn(bool flag);
    // Answer true if conductivity monitoring is on
    static bool isConductivityOn();

    // Flag whether turbidity is to be monitored or not
    static void setTurbidityOn(bool flag);
    // Answer true if turbidity monitoring is on
    static bool isTurbidityOn();

    // Print temperature sensor settings
    static void printTempSettings();

    // Print conductivity sensor settings
    static void printConductivitySettings();

    // Print turbidity sensor settings
    static void printTurbiditySettings();

    // Handle monitor if enabled
    static void handleMonitor();

  private:
    // Memorise whether temperature is to be monitored
    static bool tempOn;
    // Memorise whether conductivity is to be monitored
    static bool conductivityOn;
    // Memorise whether turbidity is to be monitored
    static bool turbidityOn;

    // C'tor: private; class monitor use enforced
    Monitor();
};

#endif /* MONITOR_H_ */
