/*
 * OnePointCalibrationHelper.h
 *
 * Created on: 15 Apr 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef ONEPOINTCALIBRATIONHELPER_H_
#define ONEPOINTCALIBRATIONHELPER_H_

#include "OnePointCalibration.h"
#include <Arduino.h>


class OnePointCalibrationHelper {
  public:
    // Initialise necessary parameters for a sensor with 1-point calibration,
    // preferences and Function Pointers (FP) for callback.
    void init(
      String name, const char* rawPrefId,
      const char* idealPrefId, const char* tempPrefId, const char* lastDatePrefId,
      float (*readRawFP)(), String (*formatStringFP)(float value, bool withUnit)
    );

    // Read date from preferences
    String readDate();

    // Print a formatted value
    void printFormattedLn(float value, bool withUnit);

    // Print raw and ideal values needed for calibration
    void printCalibValues();

    // Set calibration raw and ideal values and store them
    void setCalibValues(float ideal);

    // Answer current float value
    float readCalibratedValue();

    // Answer string of current value with unit (= true) or not
    String readString(bool withUnit);

    // Write date to preferences if not already written
    void writeDate();

    // Write to preferences
    void writeCalibTemp(float temp);

    // Write to preferences
    void writeCalib(float raw, float ideal);

    // Allow setting of calibration values for testing reasons
    void fakesetCalibValues(float raw, float ideal, float temp);

  private:
    // Sensor name
    String name;

    // Preferences ids
    const char* rawPrefId;
    const char* idealPrefId;
    const char* tempPrefId;
    const char* lastDatePrefId;

    // The sensor's one point calibration
    OnePointCalibration calibration;

    // Pointer to specific readRaw function
    float (*readRawFP)();
    // Pointer to specific formatString function
    String (*formatStringFP)(float value, bool withUnit);

    void printSensorRaw();
    void printSensorIdeal();
    void printTemp();
};

#endif /* ONEPOINTCALIBRATIONHELPER_H_ */
