/*
 * Precalibration.h
 *
 * Created on: 9 Jun 2023
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef PRECALIBRATION_H_
#define PRECALIBRATION_H_

#include "OnePointCalibration.h"
#include <Arduino.h>
#include "SD.h"


/*
 * This class is typically used the very first time an ESP32 is flashed.
 * It serves to precalibrate sensor values defined in the PRECONFIG_FILE
 * located in the root directiry of the SD card.
*/
class Precalibration {
  public:
    // Set the containg values in file PRECONFIG_FILE as preferences.
    // File format:
    // #   comment out line; *must* be on leftmost line position
    // empty line  or  line only containing spaces allowed
    // exactly 6 values, each on its own line; cp. order and meaning in the
    // following example:
    //   # Precalibration values read in after start-up.
    //   # This file will be deleted after precalibration.
    //   # A copy can be found in /wbox on this SD card.
    //   
    //   # low temperature sensor value:
    //   0
    //   # high temperature sensor value:
    //   20
    //      
    //   # conductivity sensor value:
    //   420
    //   # at a sensor temperature of:
    //   25
    //   
    //   # low turbidity sensor value:
    //   0
    //   # high turbidity sensor value:
    //   3.1
    //   
    static void setInitialSensorPrefsCond();

  private:
    // List root directory. For debugging reasons only.
    static void listRootDir(fs::FS &fs, const char* dirname);

    // Test if file PRECONFIG_FILE exists in root directory
    static boolean doesPrecalibFileExist();

    // Copy precalibration values from PRECONFIG_FILE to preferences
    static void precalibFromFile();

    // Delete the PRECONFIG_FILE. This prevents new precalibration
    // in the future.
    static void deletePrecalibFile();

    // Specific name of the precalibration file
    static const String PRECALIB_FILE;
};

#endif /* PRECALIBRATION_H_ */
