/*
 * TemperatureTest.h
 *
 * Created on: 9 Oct 2022
 * Author: ruedi.mueller@fhnw.ch
 */

// OneWire DS18S20, DS18B20, DS1822 Temperature Example without using library
// DallasTemperature.
// Source: http://www.esp32learning.com/code/esp32-and-ds18b20-temperature-sensor-example.php

#ifndef TEMPERATURETEST_H_
#define TEMPERATURETEST_H_

#include <OneWire.h>
#include "Config.h"
 
OneWire ds(Config::TEMP_PIN);         // on pin 21 (a 4.7K resistor is necessary)
 

void printTemp() {
  uint8_t data[12];
  uint8_t addr[8];
   
  for (int i = 0; i < 10; ++i) {
    if (!ds.search(addr)) {
      ds.reset_search();
      Config::outStream->print(F("."));
      delay(250);
    }
    else   break;
  }
   
  if (OneWire::crc8(addr, 7) != addr[7]) {
    Config::outStream->println(F("CRC is not valid!"));
    return;
  }
   
  // The first ROM byte indicates which chip
  int type_s;
  switch (addr[0]) {
    case 0x10:
      type_s = 1;
      break;
    case 0x28:
      type_s = 0;
      break;
    case 0x22:
      type_s = 0;
      break;
    default:
      Config::outStream->println(F("Device is not a DS18x20 family device."));
      return;
  }
   
  ds.reset();
  ds.select(addr);
  ds.write(0x44, 1);     // Start conversion, with parasite power on at the end
  delay(200);

  ds.reset();
  ds.select(addr);
  ds.write(0xBE);        // Read Scratchpad

  for (int i = 0; i < 9; i++) {
    data[i] = ds.read();
  }
   
  // Convert the data to actual temperature
  int16_t raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3;                               // 9 bit resolution default
    if (data[7] == 0x10) {
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  }
  else {
    byte cfg = (data[4] & 0x60);
    if (cfg == 0x00)   { raw = raw & ~7; }        // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20)   { raw = raw & ~3; }   // 10 bit res, 187.5 ms
    else if (cfg == 0x40)   { raw = raw & ~1; }   // 11 bit res, 375 ms
  }

  float celsius = (float)raw / 16.0;
  Config::outStream->print(F("Temperature = "));
  Config::outStream->print(celsius);
  Config::outStream->println(F("°C"));
}

#endif /* TEMPERATURETEST_H_ */
