/*
 * TwoPointCalibration.h
 *
 * Created on: 27 Feb 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef TWOPOINTCALIBRATION_H_
#define TWOPOINTCALIBRATION_H_


class TwoPointCalibration {
  public:
    // Initialise object with preference strings to calibrate the sensor
    void init(
      const char* rawLowId, const char* rawHighId,
      const char* idealLowId, const char* idealHighId);

    // Correct read (raw) value from sensor with calibrated (ideal) values
    float calculateValue(float rawValue);

    // Getters
    float getRawLow();
    float getRawHigh();
    float getIdealLow();
    float getIdealHigh();

    // Setters
    void writeRawLow(const char* prefId, float value);
    void writeIdealLow(const char* prefId, float value);
    void writeRawHigh(const char* prefId, float value);
    void writeIdealHigh(const char* prefId, float value);

  private:
    // Calibration values from preferences
    // Cp. calculateValue(float rawValue) implementation
    float rawLow;
    float rawHigh;
    float idealLow;
    float idealHigh;
};

#endif /* TWOPOINTCALIBRATION_H_ */
