/*
 * TwoPointCalibrationHelper.h
 *
 * Created on: 2 Apr 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef TWOPOINTCALIBRATIONHELPER_H_
#define TWOPOINTCALIBRATIONHELPER_H_

#include "TwoPointCalibration.h"
#include <Arduino.h>


class TwoPointCalibrationHelper {
  public:
    // Initialise necessary parameters for a sensor with 2-point calibration,
    // preferences and Function Pointers (FP) for callback.
    void init(
      String name, const char* rawLowPrefId, const char* rawHighPrefId,
      const char* idealLowPrefId, const char* idealHighPrefId, const char* lastDatePrefId,
      float (*readRawFP)(), String (*formatStringFP)(float value, bool withUnit)
    );

    // Read date from preferences
    String readDate();

    // Print a formatted value
    void printFormattedLn(float value, bool withUnit);

    // Print raw and ideal values needed for calibration
    void printCalibValues();

    // Print (analog digital converted) raw and ideal values (like pH)
    void printMixedCalibValues();

    // Set calibration low raw and ideal values and store them in preferences
    void setCalibLowValues(float idealLow);

    // Set calibration (analog digital converted) raw and ideal low values   // TODO: used?
    void setMixedCalibLowValues(float idealLow);

    // Set calibration high raw and ideal values and store them in preferences
    void setCalibHighValues(float idealHigh);

    // Set calibration (analog digital converted) raw and ideal high values   // TODO: used?
    void setMixedCalibHighValues(float idealHigh);

    // Answer ideal value
    float readCalibratedValue();

    // Answer string of current value with unit (= true) or not
    String readString(bool withUnit);

    // Write date to preferences
    void writeDate();

    // Write to preferences and calibration variable, low values
    void writeCalibLow(float rawLow, float idealLow);

    // Write to preferences and calibration variable, high values
    void writeCalibHigh(float rawHigh, float idealHigh);

    // Allow setting of calibration values for testing reasons
    void fakesetCalibLowValues(float rawLow, float idealLow);
    void fakesetCalibHighValues(float rawHigh, float idealHigh);

    // Allow setting of calibration (adc and units) values for testing reasons
    void fakesetMixedCalibLowValues(float rawLow, float idealLow);
    void fakesetMixedCalibHighValues(float rawHigh, float idealHigh);

  private:
    // Sensor name
    String name;

    // Preferences ids
    const char* rawLowPrefId;
    const char* rawHighPrefId;
    const char* idealLowPrefId;
    const char* idealHighPrefId;
    const char* lastDatePrefId;

    // The sensor's two point calibration
    TwoPointCalibration calibration;

    // Pointer to specific readRaw function
    float (*readRawFP)();
    // Pointer to specific formatString function
    String (*formatStringFP)(float value, bool withUnit);

    // Print low values
    void printSensorRawLow();
    void printSensorIdealLow();

    // Print analog digital converted low value without units
    void printSensorAdcRawLow();

    // Print high values
    void printSensorRawHigh();
    void printSensorIdealHigh();

    // Print analog digital converted high value without units
    void printSensorAdcRawHigh();
};

#endif /* TWOPOINTCALIBRATIONHELPER_H_ */
