/*
 * WbLCD.h
 *
 * Created on: 22 May 2021
 * Author: ruedi.mueller@fhnw.ch
 *
 * Capacitive Touch Breakout:
 * https://www.bastelgarage.ch/kapazitiver-touch-sensor-taster-ttp223
 * (Top View)
 *                      +--------------+
 *    3.3 V  -----------|              |
 *    GPIO #4 ----------|      TOUCH   |
 *    GND     ----------|              |
 *                      +--------------+
 */

#ifndef WBLCD_H_
#define WBLCD_H_

#include "LiquidCrystal_I2C.h"
#include "Timeout.h"


class WbLCD {
  public:
    // Init LCD display
    static void begin();

    // Answer pointer to object
    static LiquidCrystal_I2C* getLcd();

    // Print error message to lcd
    static void outLcdPrintError(String str);

    static void backlightOn();

    static void backlightOff();

    // Check in polling mode (in loop()) whether backlight should be switched off
    static void backlightCheck();

  private:
    // LCD display 20x4:
    // DUE TO A FLAW IN THE LIBRARY LiquidCrystal THE LCD OBJECT IS TO BE INSTANTIATED
    // IN SETUP RATHER THAN DURING CLASS LOADING (by Dec 2021)
    // Cp. https://forum.arduino.cc/t/problem-with-liquidcrystal-and-latest-esp32-core/892690
    static LiquidCrystal_I2C* lcd;

    // Special characters: arrow, wifi_symbol, power_off
    static const byte ARROW[];
    static const byte WIFI_SYMBOL[];
    static const byte POWEROFF_SYMBOL[];

    static Timeout timer;
};

#endif /* WBLCD_H_ */
