/*
 * WbPreferences.h
 *
 * Created on: 23 Nov 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef WBPREFERENCES_H_
#define WBPREFERENCES_H_

#include <Arduino.h>
#include <Preferences.h>


// Default values
// Default preference: ap   comment/uncomment for different default preference
#define NETWORK_METHOD 1
#define ACCESSPOINT_METHOD 2
// Default preference: net   comment/uncomment for different default preference
// #define ACCESSPOINT_METHOD 1
// #define NETWORK_METHOD 2

#define SERIAL1_ENABLED 1 
#define SERIAL1_DISABLED 2 

/*
 * WbPreferences deals with non-volatile values in a key - value structure.
 */
class WbPreferences {
  public:
    static void printPrefs();
    static void printWifiMethod();
    static void printNetCredentials();
    static void printApCredentials();
    static void printWifi();
    static void clearPrefs();

    // Timer minimum interval used for monitor ad logger
    static unsigned int getTimerMinInterval();
    static void setTimerMinInterval(unsigned int seconds);

    // Lcd filename used in LCD display mode
    static String getLcdLogfileName();
    static void setLcdLogfileName(String filename) ;

    // Lcd timer interval used in LCD display mode
    static unsigned int getLcdTimerInterval();
    static void setLcdTimerInterval(unsigned int seconds);
    // Lcd backlight timeout
    static int getBacklightTimeout();
    static void setBacklightTimeout(int minutes);

    // Network credentials
    static String getNetSsid();
    static void setNetSsid(String ssid);
    static String getNetPwd();
    static void setNetPwd(String pwd);
    // Access Points credentials
    static String getApSsid();
    static void setApSsid(String ssid);
    static String getApPwd();
    static void setApPwd(String pwd);
    // Network or access point
    static int getWifiMethod();
    static void setWifiMethod(int method);
    // Serial1 enable/disable switch
    static int getSerial1Switch();
    static void setSerial1Switch(int enabled);
    // Values for calibration
    static float getFloatValue(const char* key, const float defaultValue);
    static void putFloatValue(const char* key, float value);
    // Date for calibration
    static String getDateString(const char* id);
    static void setDateString(const char* id, String date);

  private:
    // Open namespace
    static void start();

    static unsigned int getInt(const char* key, const int defaultValue);
    static void putInt(const char* key, int value);

    static unsigned int getUInt(const char* key, const unsigned int defaultValue);
    static void putUInt(const char* key, unsigned int value);

    static String getString(const char* key, String defaultStr);
    static void putString(const char* key, String str);

    // Encapsulated preferences
    static Preferences pref;
};

#endif /* WBPREFERENCES_H_ */
