/*
 * WbServer.h
 *
 * Created on: 29 Mar 2022
 * Author: ruedi.mueller@fhnw.ch
 */

#ifndef WBSERVER_H_
#define WBSERVER_H_

#include <Arduino.h>
#include <SPIFFS.h>
#include <ESPAsyncWebServer.h>
#include <ArduinoJson.h>
#include <Wire.h>
#include "RTClib.h"
#include "Streaming.h"
#include "StopWatch.h"

#define HTTP_PORT 80


class WbServer {
  public:
    static void begin();
    static void cleanupClients();
    static void notifyClients();
    static void interpret();
    static void sendEvent(String sensorReading, String id, long millis);

    static bool active;
    static void updateReadingsIfActive();
    static unsigned long lastReading;

  private:
    static void initWiFiMethod();
    static void initSPIFFS();
    static void onRootRequest(AsyncWebServerRequest* request);
    static String processor(const String& var);
    static String readingsProcessor(const String& var);
    static void onDirRequest(AsyncWebServerRequest* request);
    static void initWebServer();
    static void handleWebSocketMessage(void* arg, uint8_t* data, size_t len);
    static void onEvent(AsyncWebSocket* server, AsyncWebSocketClient* client,
                        AwsEventType type, void* arg, uint8_t* data, size_t len);
    static void onEventConnect();
    static void initWebSocket();
    static void addDateTime(File file);
    static void addConfirmFunction(String filename, int num);
    static void listRootDir(fs::FS &fs, const char* dirname);
    static void fileDownload(AsyncWebServerRequest* request);
    static void fileDelete(AsyncWebServerRequest* request);
    static void terminate(AsyncWebServerRequest* request);
    static void pageNotFound(AsyncWebServerRequest* request);

    static AsyncWebServer server;
    static AsyncWebSocket ws;
    static IPAddress myIP;
    static AsyncEventSource events;

    // String containing the command from the website
    static String webCommand;

    static String webpage;
    static const char tablestart_html[];
    static const char terminate_html[];
    static const char readings_html[];
    static bool readingsPageActive;
    static StopWatch stopWatch;

    // Readings
    static String temp;
    static String cond;
    static String turb;
};

#endif /* WBSERVER_H_ */
