/*
 * Conductivity.cpp
 *
 * Created on: 15 Apr 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "Conductivity.h"
#include <Arduino.h>
#include "Config.h"
#include "OnePointCalibrationHelper.h"


OnePointCalibrationHelper Conductivity::helper;

char Conductivity::buf[20];


void Conductivity::begin() {
  helper.init(
    CONDUCTIVITY_STR, COND_RAW_STR, COND_IDEAL_STR,
    COND_CALIB_TEMP_STR, COND_LAST_DATE_STR,
    readRaw, formatString);
}


// TODO still needed?
float Conductivity::voltageFormula(float voltage) {
  return (133.42 * voltage * voltage * voltage -
          255.86 * voltage * voltage +
          857.39 * voltage);
}


int Conductivity::getMedianValue(int arr[], int size) {
  int i, j, bTemp;
  for (j = 0; j < size - 1; j++) {
    for (i = 0; i < size - j - 1; i++) {
      if (arr[i] > arr[i + 1]) {
        bTemp = arr[i];
        arr[i] = arr[i + 1];
        arr[i + 1] = bTemp;
      }
    }
  }
  
  if ((size & 1) > 0) {
    bTemp = arr[(size - 1) / 2];
  }
  else {
    bTemp = (arr[size / 2] + arr[size / 2 - 1]) / 2;
  }
  return bTemp;
}


// without calibration   // TODO
float Conductivity::readTdsValue(float temp) {
  // Read median value
  const int SIZE = 5;
  int tdsValueBuffer[SIZE];
  for (int i = 0; i < SIZE; ++i) {
    tdsValueBuffer[i] = analogRead(Config::CONDUCTIVITY_PIN);
    delay(20);
  }
  // Median value read from ADC pin
  int medianValue = getMedianValue(tdsValueBuffer, SIZE);
  // Convert to voltage
  float voltage =
    medianValue * Config::CONDUCTIVITY_AREF / Config::CONDUCTIVITY_ADC_RANGE;
  // Calculate temperature compensation
  float compensationCoefficient = 1.0 + (0.02 * (temp - 25.0));
  float compensationVoltage = voltage / compensationCoefficient;
  // Convert temp-compensated voltage to tds value
  float tdsValue =
    (133.42 * compensationVoltage * compensationVoltage * compensationVoltage - 
    255.86 * compensationVoltage * compensationVoltage + 857.39 * compensationVoltage) * 
    0.5;
  return tdsValue;
}


float Conductivity::calculateFromTdsValue(float temp) {
  return 2 * readTdsValue(temp);
}


// TODO compensation, calibration
float Conductivity::readRaw() {
  // raw: converted from voltage to electrical conductivity,
  // without temperature compensation,
  // without calibration   // TODO
  float voltage =
    analogRead(Config::CONDUCTIVITY_PIN) /
    Config::CONDUCTIVITY_ADC_RANGE *
    Config::CONDUCTIVITY_AREF;

  // Without temperature compensation
  float ecValue = voltageFormula(voltage);
  return ecValue;
}


String Conductivity::formatString(float conductivity, bool withUnit) {
  // Print number rounded as an int
  int condTimesTen = (int)(10 * conductivity);
  int prePoint = condTimesTen / 10;

  sprintf(buf, "%i%s", prePoint, withUnit ? " uS/cm" : "");
  return String(buf);
}
