/*
 * ConductivityTest.cpp
 *
 * Created on: 8 Oct 2022
 * Author: ruedi.mueller@fhnw.ch
 */

#include "ConductivityTest.h"
#include "Config.h"
#include "Temperature.h"


int ConductivityTest::valueBuffer[SCOUNT];

float ConductivityTest::averageVoltage;
float ConductivityTest::tdsValue;
float ConductivityTest::temperature;


float ConductivityTest::getAverageVoltage() {
  return averageVoltage;
}

float ConductivityTest::getTdsValue() {
  return tdsValue;
}

float ConductivityTest::getTemperature() {
  return temperature;
}


// Median filtering algorithm
// A function to get a stable TDS value from an array of readings.
int ConductivityTest::getMedianNum(int array[], int iFilterLen) {
  int tab[iFilterLen];
  for (int i = 0; i < iFilterLen; i++) {
    tab[i] = array[i];
  }

  int temp;
  for (int j = 0; j < iFilterLen - 1; j++) {
    for (int i = 0; i < iFilterLen - j - 1; i++) {
      if (tab[i] > tab[i + 1]) {
        temp = tab[i];
        tab[i] = tab[i + 1];
        tab[i + 1] = temp;
      }
    }
  }
  if ((iFilterLen & 1) > 0){
    temp = tab[(iFilterLen - 1) / 2];
  }
  else {
    temp = (tab[iFilterLen / 2] + tab[iFilterLen / 2 - 1]) / 2;
  }
  return temp;
}


void ConductivityTest::measureTDS() {
  unsigned long sampleTimepoint = millis();
  int bufferIndex = 0;
  while (bufferIndex < SCOUNT) {
    // Read the analog value from the ADC at a 40 ms time period
    if (millis() - sampleTimepoint > 40U) { 
      sampleTimepoint = millis();
  
      // Read an analog value and store it into the buffer
      valueBuffer[bufferIndex] = analogRead(Config::CONDUCTIVITY_PIN); 
      bufferIndex++;
    }
  }
  averageVoltage = getMedianNum(valueBuffer, SCOUNT) * (float)VREF / 4096.0;
  
  temperature = Temperature::helper.readCalibratedValue();
  
  // Temperature compensation formula
  float compensationCoefficient = 1.0 + 0.02 * (temperature - 25.0);
  // Temperature compensation
  float compensationVoltage = averageVoltage / compensationCoefficient;
  
  // Convert voltage value to tds value
  tdsValue =
    (133.42 *compensationVoltage * compensationVoltage * compensationVoltage -
    255.86 * compensationVoltage * compensationVoltage + 
    857.39 * compensationVoltage)
    * 0.5;
}
