/*
 * Config.cpp
 *
 * Created on: 13 Feb 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "Config.h"
#include <Arduino.h>


// xxx    Change if new version    xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
const String Config::VERSION_NO PROGMEM = "V2.2.1";
// xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

const String Config::VERSION PROGMEM = "Wasserbox  " + VERSION_NO;
const String Config::INSTITUTION PROGMEM = "Fachhochschule Nordwestschweiz, Maker Studio, (c) 2021-23";

Stream* Config::inStream = &Serial;
// Print* instead of Stream* because of WbServer and WbPrintBuffer
Print* Config::outStream = &Serial;
WbPrintBuffer Config::pb;

StopWatch Config::stopWatch;

// ESP32 Pinout and connections                                                  
//    USED ESP32 PINS                       PWR            SD                  RTC, LCD
//                                     5V  HOLD   --   --  CS   --   --   --   ---I2C---
//    +------------------------------------------------------------------------------------+
//    |                       o    o    o    o    o    o    o    o    o    o    o    o     |
//    |                      BAT  EN   USB  #13  #12  #27  #33  #15  #32  #14  SCL  SDA    |
//    |                                     GPIO GPIO GPIO GPIO GPIO GPIO GPIO             |
//    |                                                                                    |
//    |                           ESP32 Feather Huzzah (Adafruit)                          |
//    |                                                                                    |
//    |                       GPIO GPIO GPI  GPI  GPI  GPIO                                |
//    |                       #26  #25  #34  #39  #36  #4                            GPIO  |
//    |   RST  3V   NC   GND  A0   A1   A2   A3   A4   A5   SCK  MO   MI   RX   TX   #21   |
//    |    o    o    o    o    o    o    o    o    o    o    o    o    o    o    o    o    |
//    +------------------------------------------------------------------------------------+
//            3.3V       GND  --   --   Cond Turb LCD  Tch   ----SPI-----  Serial1   Temp
//                                                Btn  Btn     SD card     option

const int Config::TOUCH_BUTTON_PIN = 4;
const int Config::BUTTON_PIN = 36;

const int Config::POWER_HOLD_PIN = 13;       // also connected to on-board LED

const int Config::SD_CS_PIN = 33;            // defined by adalogger card wiring

const int Config::UNUSED_A0_PIN = A0;
const int Config::UNUSED_A1_PIN = A1;
const int Config::UNUSED_27_PIN = 27;
const int Config::UNUSED_12_PIN = 12;
const int Config::UNUSED_14_PIN = 14;
const int Config::UNUSED_15_PIN = 15;
const int Config::UNUSED_32_PIN = 32;

const uint8_t Config::TEMP_PIN = 21;
const int Config::CONDUCTIVITY_PIN = A2;
const int Config::TURBIDITY_PIN = A3;


// CONSTANTS
const String Config::WEB_TITLE PROGMEM = "Waterbox Command and File Server 1.0";

// CAUTION: Must be a name with format "cccccc00.csv", i.e. consisting
// of 1 to 6 characters 'c' followed by "00.csv"
// Examples: "l00.csv", "hello00.csv", "data00.csv"
const char Config::DEFAULT_LOGFILE_NAME[] PROGMEM = "noname.csv";
const String Config::LCD_DEFAULT_LOGFILE_NAME PROGMEM = "lcd00.csv";

const char Config::DAYS_OF_WEEK[] PROGMEM = "MonTueWedThuFriSatSun";

const String Config::DATE_HEADER PROGMEM = "date [dd.mm.yyyy]";
const String Config::TIME_HEADER PROGMEM = "time [hh:mm:ss]";
const String Config::TEMP_HEADER PROGMEM = "temperature [degC]";
const String Config::CONDUCTIVITY_HEADER PROGMEM = "conductivity [uS/cm]";
const String Config::TURBIDITY_HEADER PROGMEM = "turbidity [cm]";

const float Config::CONDUCTIVITY_ADC_RANGE = 4096.0;
const float Config::CONDUCTIVITY_AREF = 3.3;
const float Config::CONDUCTIVITY_CALIB_DEFAULT_TEMP = 25.0;

const int Config::TIMER_MIN_INTERVAL = 5;
const int Config::LCD_TIMER_INTERVAL = 10;

const long Config::MAX_RESPONSETEXT_LENGTH = 20000UL;


void Config::setPinMode() {
  pinMode(Config::UNUSED_A0_PIN, INPUT_PULLDOWN);
  pinMode(Config::UNUSED_A1_PIN, INPUT_PULLDOWN);
  pinMode(Config::UNUSED_27_PIN, INPUT_PULLDOWN);
  pinMode(Config::UNUSED_12_PIN, INPUT_PULLDOWN);
  pinMode(Config::UNUSED_14_PIN, INPUT_PULLDOWN);
  pinMode(Config::UNUSED_15_PIN, INPUT_PULLDOWN);
  pinMode(Config::UNUSED_32_PIN, INPUT_PULLDOWN);
}
