/*
 * LcdButton.cpp
 *
 * Created on: 7 May 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "LcdButton.h"
#include <Arduino.h>
#include "Config.h"
#include "Streaming.h"
#include "WbServer.h"
#include "LcdButtonTimer.h"
#include "LcdStatesDiagram.h"
#include "LcdText.h"


volatile bool LcdButton::changed = false;
volatile bool LcdButton::down = false;
volatile bool LcdButton::up = true;


// ISR function
// ============
// IRAM_ATTR flag to store the function code in RAM (and not in Flash)
// to run faster.
void IRAM_ATTR LcdButton::btnIsr() {
  int downCount = 0;
  int upCount = 0;
  for (int i = 0; i < 10000; ++i) {
    if (digitalRead(Config::BUTTON_PIN) == HIGH) {   // Pulldown-R version
      downCount += 1;
    }
    if (digitalRead(Config::BUTTON_PIN) == LOW) {    // Pulldown-R version
      upCount += 1;
    }
  }
  if (downCount > 9900) {
    changed = true;
    down = true;
    up = false;
  }
  if (upCount > 9900) {
    changed = true;
    down = false;
    up = true;
  }
}


void LcdButton::begin() {
  pinMode(Config::BUTTON_PIN, INPUT);  // External Pullup used
  enable();
  changed = false;
  down = false;
  up = true;
  Serial << F("Button irq attached\n");
}


void LcdButton::enable() {
  // Enable interrupt
  attachInterrupt(digitalPinToInterrupt(Config::BUTTON_PIN), btnIsr, CHANGE);   // CHANGE 1.5.0
}


void LcdButton::disable() {
  // Disable interrupt
  detachInterrupt(Config::BUTTON_PIN);
}


void LcdButton::setChanged(bool flag) {
  changed = flag;
}

void LcdButton::setDown(bool flag) {
  down = flag;
}

void LcdButton::setUp(bool flag) {
  up = flag;
}

bool LcdButton::hasChanged() {
  return changed;
}

bool LcdButton::isDown() {
  changed = false;
  return down;
}

bool LcdButton::isUp() {
  changed = false;
  return up;
}


void LcdButton::powerOffIfWbServerActive() {
  if (WbServer::active) {
    if (digitalRead(Config::BUTTON_PIN) == HIGH) {
      if (LcdButtonTimer::haveElapsedEqualMore(10000)) {
        LcdStatesDiagram::poweroff();
      }
      else if (LcdButtonTimer::haveElapsedEqualMore(9000)) {
        LcdText::powerOffSymbol();
      }
      else if (LcdButtonTimer::haveElapsedEqualMore(8000)) {
        LcdText::wifiNoSymbol();
      }
      else if (LcdButtonTimer::haveElapsedEqualMore(7000)) {
        LcdText::powerOffSymbol();
      }
      else if (LcdButtonTimer::haveElapsedEqualMore(6000)) {
        LcdText::wifiNoSymbol();
      }
      else if (LcdButtonTimer::haveElapsedEqualMore(5000)) {
        LcdText::powerOffSymbol();
      }
      else if (LcdButtonTimer::haveElapsedEqualMore(4000)) {
        LcdText::wifiNoSymbol();
      }
      else if (LcdButtonTimer::haveElapsedEqualMore(3000)) {
        LcdText::powerOffSymbol();
      }
    }
    else {
      LcdButtonTimer::start();
      LcdText::wifiNoSymbol();
    }
  }
}
