/*
 * FSM.cpp
 *
 * Created on: 8 May 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "LcdFSM.h"
#include "Streaming.h"


const int LcdFSM::NO_CONDITION = -1;

LcdState* LcdFSM::currentState;
int (* LcdFSM::conditionFunction)();
unsigned long LcdFSM::startMillis;


void LcdFSM::begin(LcdState* currentState) {
  LcdFSM::currentState = currentState;
}


void LcdFSM::nextState() {
  int condition = conditionFunction();

  if (condition == LcdFSM::NO_CONDITION) {
    // Unconditional transition
    // State MUST ONLY be connected to one (other) state
    currentState->performAction(0);
    currentState = currentState->getConnectedStates(0);
  }
  else {
    // Conditional transition
    for (unsigned int i = 0; i < currentState->arrSize(); ++i) {
      if (currentState->getConditions()[i] == condition) {
        printCurrentState();
        printElapsedMillis();
        currentState->performAction(i);
        currentState = currentState->getConnectedStates(i);
        // Serial << F("next state: ") << currentState->getName() << endl;   // DEBUG
        // printElapsedMillis();   // DEBUG
        return;
      }
    }
  }
}


void LcdFSM::printCurrentState() {
  Serial << F("currentState: ");
  #ifndef NAMELESS_STATE
  Serial << currentState->getName() << endl;
  #else
  Serial << F("Nameless mode! Change to named mode.\n"));
  #endif
}


void LcdFSM::setConditionFunction(int (*conditionFunction)()) {
  LcdFSM::conditionFunction = conditionFunction;
}


LcdState* LcdFSM::getCurrentState() {
  return currentState;
}


// Millis functions
void LcdFSM::printMillis() {
  Serial << F("millis(): ") << millis() << endl;
}


void LcdFSM::printElapsedMillis() {
  Serial << F("elapsed: ") << (millis() - startMillis) << endl;
}


void LcdFSM::resetMillis() {
  startMillis = millis();
}


// State transition condition function
// Set FSM to be time sensitive.
int LcdFSM::millisCondFunction() {
  return (millis() - startMillis) / 10 * 10;
}
