/*
 * LcdSensorValuesCollector.cpp
 *
 * Created on: 23 May 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "LcdSensorValuesCollector.h"
#include "LcdText.h"
#include "LcdStatesDiagram.h"
#include "Temperature.h"
#include "Conductivity.h"
#include "Turbidity.h"
#include "Streaming.h"


#define TEMP_STATE 0
#define CONDUCTIVITY_STATE 1
#define TURBIDITY_STATE 2
#define NUM_OF_STATES 3

#define NUM_CHARS 20


int LcdSensorValuesCollector::state = TEMP_STATE;
// Initial value strings unknown; values have to be collected first
String LcdSensorValuesCollector::temp = "??.?";
String LcdSensorValuesCollector::conductivity = "???";
String LcdSensorValuesCollector::turbidity = "???.?";


void LcdSensorValuesCollector::collect() {
  // Time critical activities
  switch (state) {
    case TEMP_STATE:
      readSetTemp();
      break;
    case CONDUCTIVITY_STATE:
      readSetConductivity();
      break;
    case TURBIDITY_STATE:
      readSetTurbidity();
      break;
  }
  state = (state + 1) % NUM_OF_STATES;
  LcdStatesDiagram::collectOff();
}


String LcdSensorValuesCollector::valuesStringUpper() {
  //           "01234567890123456789"
  //           "  °C | uS/cm |    cm"
  return "  \xDF\x43 | \xE4S/cm |    cm";
}


String LcdSensorValuesCollector::spaces(int n) {
  String s = "";
  for (int i = 0; i < n; ++i) {
    s += " ";
  }
  return s;
}


String LcdSensorValuesCollector::valuesStringLower() {
  //           "01234567890123456789"
  //           "  °C | uS/cm |    cm"
  //           "22.3 |   345 | 107.2"
  int tempLen = temp.length();
  int condLen = conductivity.length();
  int turbLen = turbidity.length();
  if (tempLen > 4)   temp = "--.-";
  if (condLen > 3)   conductivity = "---";
  if (turbidity.toInt() == (int)Turbidity::TOO_HIGH ||
      turbidity.toInt() == (int)Turbidity::TOO_LOW)
  {
    turbidity = "---.-";
    turbLen = 5;
  }
  return
    spaces(4 - tempLen) + temp + " |   " +
    spaces(3 - condLen) + conductivity + " | " +
    spaces(5 - turbLen) + turbidity;
}


void LcdSensorValuesCollector::readSetTemp() {
  temp = Temperature::helper.readString(false);
}


void LcdSensorValuesCollector::readSetConductivity() {
  conductivity = Conductivity::helper.readString(false);
}


void LcdSensorValuesCollector::readSetTurbidity() {
  turbidity = Turbidity::readString(false);
}
