/*
 * LcdText.cpp
 *
 * Created on: 23 May 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "LcdText.h"
#include <LiquidCrystal_I2C.h>
#include "LcdFSM.h"
#include "Config.h"
#include "LcdSensorValuesCollector.h"
#include "LcdStatesDiagram.h"
#include "LcdLogger.h"
#include "WbRTC.h"
#include "WbLCD.h"
#include "WbSD.h"
#include "Streaming.h"
#include "WbServer.h"
#include "WbPreferences.h"
#include "StopWatch.h"
#include "LcdButtonTimer.h"
#include "LcdButton.h"


// Choose desired language on the lcd display, or output to SERIAL
#define GERMAN_TEXT
// #define ENGLISH_TEXT
// #define SERIAL_TEXT


// LCD display lines
#define LINE0 0
#define LINE1 1
#define LINE2 2
#define LINE3 3
#define SCREEN_WIDTH 20


int LcdText::patternPos = 0;
bool LcdText::erased = false;
bool LcdText::written = false;
bool LcdText::once2 = false;
bool LcdText::once1 = false;
bool LcdText::once0 = false;
LiquidCrystal_I2C* LcdText::wbLcd = nullptr;


void LcdText::resetOnces() {
  once2 = once1 = once0 = false;
}


#ifndef SERIAL_TEXT
void LcdText::printClearLastLine() {
  wbLcd->setCursor(0, LINE3);
  //             "01234567890123456789"
  wbLcd->print(F("                    "));
}
#endif
#ifdef SERIAL_TEXT
void LcdText::textBorder() {
  //          "+01234567890123456789+"
  Serial << F("+--------------------+\n");
}
void LcdText::printClearLastLine() {
  //           "|01234567890123456789|"
  Serial << F("|                    |\n");
  textBorder();
}
#endif


#ifdef  GERMAN_TEXT
void LcdText::powerOnScreen() {
  wbLcd = WbLCD::getLcd();
  wbLcd->home();
  wbLcd->print(F(" "));
  wbLcd->print(Config::VERSION);
  wbLcd->setCursor(0, LINE1);
  //             "01234567890123456789"
  //             " www.wassernetz.ch  "
  //             " Bitte Taste 5 Sek. "
  //             " gedrückt halten.   "
  wbLcd->print(F(" www.wassernetz.ch  "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F(" Bitte Taste 5 Sek. "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F(" gedr\xF5\x63kt halten.  "));
  for (int i = 4; i > 0; --i) {
    delay(1000);
    wbLcd->setCursor(13, LINE2);
    wbLcd->print(i);
    Serial << i << ".. ";
  }
  delay(1000);
  // Power on ESP32 even after release of button
  pinMode(Config::POWER_HOLD_PIN, OUTPUT);
  digitalWrite(Config::POWER_HOLD_PIN, HIGH);
  Serial << F("Power hold circuit activated\n");
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::powerOnScreen() {
  wbLcd = WbLCD::getLcd();
  wbLcd->home();
  wbLcd->print(F(" "));
  wbLcd->print(Config::VERSION);
  wbLcd->setCursor(0, LINE1);
  //             "01234567890123456789"
  //             " www.wassernetz.ch  "
  //             " Please keep button "
  //             " pressed for 5 sec. "
  wbLcd->print(F(" www.wassernetz.ch  "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F(" Please keep button "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F(" pressed for 5 sec. "));
  for (int i = 4; i > 0; --i) {
    delay(1000);
    wbLcd->setCursor(13, LINE3);
    wbLcd->print(i);
    Serial << i << ".. ";
  }
  delay(1000);
  // Power on ESP32 even after release of button
  pinMode(Config::POWER_HOLD_PIN, OUTPUT);
  digitalWrite(Config::POWER_HOLD_PIN, HIGH);
  Serial << F("Power hold circuit activated\n");
}
#endif
#ifdef SERIAL_TEXT
void LcdText::powerOnScreen() {
  textBorder();
  //          "|01234567890123456789|"
  //          "| Wasserbox  Vx.y.z  |"
  Serial << F("| ") << Config::VERSION << F("  |\n");
  Serial << F("| www.wassernetz.ch  |\n");
  Serial << F("| Bitte Taste 5 Sek. |\n");
  Serial << F("| gedr\xF5\x63kt halten.  |\n");
  textBorder();
  for (int i = 4; i > 0; --i) {
    delay(1000);
    Serial << i << ".. ";
  }
  delay(1000);
  // Power on ESP32 even after release of button
  pinMode(Config::POWER_HOLD_PIN, OUTPUT);
  digitalWrite(Config::POWER_HOLD_PIN, HIGH);
  Serial << F("Power hold circuit activated\n");
}
#endif


#ifdef GERMAN_TEXT
void LcdText::pSplashScreen() {
  wbLcd = WbLCD::getLcd();
  wbLcd->home();
  wbLcd->print(F(" "));
  wbLcd->print(Config::VERSION);
  wbLcd->setCursor(0, LINE1);
  //             "01234567890123456789"
  //             " www.wassernetz.ch  "
  //             "     Guten Tag!     "
  //             "Taste erneut drücken"
  wbLcd->print(F(" www.wassernetz.ch  "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("     Guten Tag!     "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F("Taste erneut dr\xF5\x63ken"));
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pSplashScreen() {
  wbLcd = WbLCD::getLcd();
  wbLcd->home();
  wbLcd->print(F(" "));
  wbLcd->print(Config::VERSION);
  wbLcd->setCursor(0, LINE1);
  //             "01234567890123456789"
  wbLcd->print(F(" www.wassernetz.ch  "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("  Nice to meet you  "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F(" Press button again "));
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pSplashScreen() {
  textBorder();
  //          "|01234567890123456789|"
  //          "| Wasserbox  V0.7.8  |"
  Serial << F("| ") << Config::VERSION << F("  |\n");
  Serial << F("| www.wassernetz.ch  |\n");
  Serial << F("|     Guten Tag!     |\n");
  Serial << F("|Taste erneut drücken|\n");
  textBorder();
}
#endif


#ifndef SERIAL_TEXT
void LcdText::printValuesStrings() {
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(LcdSensorValuesCollector::valuesStringUpper());
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(LcdSensorValuesCollector::valuesStringLower());
}
#endif
#ifdef SERIAL_TEXT
void LcdText::printValuesStrings() {
  Serial << F("|") << LcdSensorValuesCollector::valuesStringUpper() << F("|\n");
  Serial << F("|") << LcdSensorValuesCollector::valuesStringLower() << F("|\n");
  textBorder();
}
#endif


#ifdef GERMAN_TEXT
void LcdText::printDateTime() {
  char buf[30];
  // Save current time
  WbRTC::saveCurrentDateTime();
  int n = WbRTC::savedDayOfTheWeek();
  char daysOfName[] = "MoDiMiDoFrSaSo";
  char dayName[] = { daysOfName[2*n], daysOfName[2*n + 1], '\0' };
  wbLcd->home();
  //           "01234567890123456789"
  //           "Mo, 12.12.2021      "
  sprintf(buf, "%s, %2d.%02d.%4d      ",
    dayName, WbRTC::savedDay(), WbRTC::savedMonth(), WbRTC::savedYear()
  );
  wbLcd->print(buf);
  wbLcd->setCursor(0, LINE1);
  //           "01234567890123456789"
  //           "17:33:37            "
  sprintf(buf, "%02d:%02d:%02d            ",
    WbRTC::savedHour(), WbRTC::savedMinute(), WbRTC::savedSecond()
  );
  wbLcd->print(buf);
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::printDateTime() {
  char buf[30];
  // Save current time
  WbRTC::saveCurrentDateTime();
  int n = WbRTC::savedDayOfTheWeek();
  char daysOfName[] = "MonTueWedThuFriSatSun";
  char dayName[] = { daysOfName[3*n], daysOfName[3*n + 1], daysOfName[3*n + 2], '\0' };
  wbLcd->home();
  //           "01234567890123456789"
  //           "Mon, 15/07/2021     "
  sprintf(buf, "%s, %2d/%02d/%4d     ",
    dayName, WbRTC::savedDay(), WbRTC::savedMonth(), WbRTC::savedYear()
  );
  wbLcd->print(buf);
  wbLcd->setCursor(0, LINE1);
  //           "01234567890123456789"
  //           "17:33:37            "
  sprintf(buf, "%02d:%02d:%02d            ",
    WbRTC::savedHour(), WbRTC::savedMinute(), WbRTC::savedSecond()
  );
  wbLcd->print(buf);
}
#endif
#ifdef SERIAL_TEXT
void LcdText::printDateTime() {
  char buf[30];
  textBorder();
  // Save current time
  WbRTC::saveCurrentDateTime();
  int n = WbRTC::savedDayOfTheWeek();
  char daysOfName[] = "MoDiMiDoFrSaSo";
  char dayName[] = { daysOfName[2*n], daysOfName[2*n + 1], '\0' };
  //           "|01234567890123456789|"
  //           "|Mo, 15.07.2021      |"
  sprintf(buf, "|%s, %2d.%02d.%4d      |\n",
    dayName, WbRTC::savedDay(), WbRTC::savedMonth(), WbRTC::savedYear()
  );
  Serial << buf;
  //           "|17:33:37            |"
  sprintf(buf, "|%02d:%02d:%02d            |\n",
    WbRTC::savedHour(), WbRTC::savedMinute(), WbRTC::savedSecond()
  );
  Serial << buf;
}
#endif


void LcdText::pLgReadyScreen1() {
  pLgPwrOffScreen1();
  LcdFSM::resetMillis();
}


void LcdText::pLgReadyScreen2() {
  pLgPwrOffScreen2();
  LcdFSM::resetMillis();
}


void LcdText::pLgReadyScreen3() {
  pLgPwrOffScreen3();
  LcdFSM::resetMillis();
}


#ifdef GERMAN_TEXT
void LcdText::pLgPwrOffScreen1() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "Für Aufzeichnung    "
  //             "Taste kurz drücken. "
  wbLcd->print(F("F\xF5r Aufzeichnung    "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("Taste kurz dr\xF5\x63ken. "));
  printValuesStrings();
  LcdStatesDiagram::collectOn();
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pLgPwrOffScreen1() {
  wbLcd->home();
  //             "01234567890123456789"
  wbLcd->print(F("To record: briefly  "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("press the button.   "));
  printValuesStrings();
  LcdStatesDiagram::collectOn();
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pLgPwrOffScreen1() {
  textBorder();
  //          "|01234567890123456789|"
  Serial << F("|Für Aufzeichnung    |\n");
  Serial << F("|Taste kurz drücken. |\n");
  printValuesStrings();
  LcdStatesDiagram::collectOn();
}
#endif


void LcdText::pLgPwrOffScreen2() {
  printDateTime();
  printValuesStrings();
  LcdStatesDiagram::collectOn();
}


#ifdef GERMAN_TEXT
void LcdText::pLgPwrOffScreen3() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "Für Ausschalten     "
  //             "Taste lang drücken. "
  wbLcd->print(F("F\xF5r Ausschalten     "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("Taste lang dr\xF5\x63ken. "));
  printValuesStrings();
  LcdStatesDiagram::collectOn();
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pLgPwrOffScreen3() {
  wbLcd->home();
  //             "01234567890123456789"
  wbLcd->print(F("To power off: press "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("button >8 seconds   "));
  printValuesStrings();
  LcdStatesDiagram::collectOn();
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pLgPwrOffScreen3() {
  textBorder();
  //          "|01234567890123456789|"
  Serial << F("|Für Ausschalten     |\n");
  Serial << F("|Taste lang drücken. |\n");
  printValuesStrings();
  LcdStatesDiagram::collectOn();
}
#endif


#ifdef GERMAN_TEXT
void LcdText::pLgWifiScreen() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "WLAN-Modus          "
  //             "SSID: ZyKEL         "
  //             "http://wbox.local   "
  //             "192.168.1.54        "
  wbLcd->print(F("WLAN-Modus          "));
  wbLcd->setCursor(0, LINE1);
  String ssid = F("SSID: ");
  ssid +=
    (WbPreferences::getWifiMethod() == NETWORK_METHOD) ? WbPreferences::getNetSsid() : WbPreferences::getApSsid();
  if (ssid.length() > SCREEN_WIDTH) {
    ssid = ssid.substring(0, SCREEN_WIDTH);
    ssid.setCharAt(SCREEN_WIDTH - 3, '.');
    ssid.setCharAt(SCREEN_WIDTH - 2, '.');
    ssid.setCharAt(SCREEN_WIDTH - 1, '.');
  }
  else {
    ssid += F("             ");
    ssid = ssid.substring(0, SCREEN_WIDTH);
  }
  wbLcd->print(ssid);
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("http://wbox.local   "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F("                    "));

  LcdStatesDiagram::setActive(false);
  LcdFSM::resetMillis();
  LcdButtonTimer::start();
  WbServer::begin();
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pLgWifiScreen() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "WiFi mode           "
  //             "SSID: ZyKEL         "
  //             "http://wbox.local   "
  //             "192.168.1.54        "
  wbLcd->print(F("WiFi mode           "));
  wbLcd->setCursor(0, LINE1);
  String ssid = F("SSID: ");
  ssid +=
    (WbPreferences::getWifiMethod() == NETWORK_METHOD) ? WbPreferences::getNetSsid() : WbPreferences::getApSsid();
  if (ssid.length() > SCREEN_WIDTH) {
    ssid = ssid.substring(0, SCREEN_WIDTH);
    ssid.setCharAt(SCREEN_WIDTH - 3, '.');
    ssid.setCharAt(SCREEN_WIDTH - 2, '.');
    ssid.setCharAt(SCREEN_WIDTH - 1, '.');
  }
  else {
    ssid += F("             ");
    ssid = ssid.substring(0, SCREEN_WIDTH);
  }
  wbLcd->print(ssid);
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("http://wbox.local   "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F("                    "));

  LcdStatesDiagram::setActive(false);
  LcdFSM::resetMillis();
  LcdButtonTimer::start();
  WbServer::begin();
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pLgWifiScreen() {
  textBorder();
  //          "|01234567890123456789|"
  //          "|WLAN-Modus          |"
  //          "|SSID: ZyKEL         |"
  //          "|http://wbox.local   |"
  //          "|192.168.1.54        |"
  Serial << F("|WLAN-Modus          |\n");
  String ssid = F("|SSID: ");
  ssid +=
    (WbPreferences::getWifiMethod() == NETWORK_METHOD) ? WbPreferences::getNetSsid() : WbPreferences::getApSsid();
  ssid += F("             ");
  ssid = ssid.substring(0, SCREEN_WIDTH);
  Serial << ssid;
  Serial << F("|http://wbox.local   |\n");
  printClearLastLine();

  LcdStatesDiagram::setActive(false);
  LcdFSM::resetMillis();
  LcdButtonTimer::start();
  WbServer::begin();
}
#endif


#ifndef SERIAL_TEXT
void LcdText::printRotatePattern() {
  wbLcd->setCursor(12, LINE0);
  //             "01234567890123456789"
  //             "WLAN-Modus  ......  "
  String rot = String(F("..   ...   ")).substring(patternPos + 5, patternPos + 11);
  wbLcd->print(rot);
  if (--patternPos < -5)   patternPos = 0;
}
void LcdText::printIP(String ip) {
  wbLcd->setCursor(12, LINE0);
  if (WbPreferences::getWifiMethod() == NETWORK_METHOD) {
    wbLcd->print(F(" (net)"));
  }
  else {
    wbLcd->print(F(" (ap) "));
  }
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(ip);
}
#endif
#ifdef SERIAL_TEXT
void LcdText::printRotatePattern() {
  Serial << F(".");
}
void LcdText::printIP(String ip) {
  Serial << endl << ip;
  Serial << ((WbPreferences::getWifiMethod() == NETWORK_METHOD) ? F(" (net)") : F(" (ap)")) << endl;
}
#endif


void LcdText::pLgRunScreenWriteFile() {
  // Start lcd display mode logging
  LcdLogger::timer.setLogDateTimeNow();
  // Create csv-file with file headers
  if (LcdLogger::createFile()) {
    LcdLogger::writeFileHeaders();
    pLgRunScreen1();
  }
  else {
    if (!WbSD::isPresent()) {
      LcdText::printSdNotPresent();
    }
    else {
      LcdText::printFileCount99();
    }
    // TODO: blocking error message to LCD and Serial
  }
}


#ifdef GERMAN_TEXT
void LcdText::pLgRunScreenWriteValues() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "Sensor-Daten werden "
  //             "gespeichert.        "
  //             "Taste inaktiv       "
  wbLcd->print(F("Sensor-Daten werden "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("gespeichert.        "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("Taste inaktiv       "));
  printClearLastLine();
  Config::stopWatch.start();
  LcdLogger::writeFileValues();

  while (!Config::stopWatch.hasReached(2000)) { }
  Config::stopWatch.stop();
  // Serial << F("Stop watch: ") << Config::stopWatch.getInterval() << endl;  // DEBUG
  pLgRunScreen1();
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pLgRunScreenWriteValues() {
  wbLcd->home();
  //             "01234567890123456789"
  wbLcd->print(F("Saving sensor data. "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("Button inactive     "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("                    "));
  printClearLastLine();
  Config::stopWatch.start();
  LcdLogger::writeFileValues();

  while (!Config::stopWatch.hasReached(2000)) { }
  Config::stopWatch.stop();
  // Serial << F("Stop watch: ") << Config::stopWatch.getInterval() << endl;  // DEBUG
  pLgRunScreen1();
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pLgRunScreenWriteValues() {
  textBorder();
  //          "|01234567890123456789|"
  Serial << F("|Sensor-Daten werden |\n");
  Serial << F("|gespeichert.        |\n");
  Serial << F("|Taste inaktiv       |\n");
  printClearLastLine();
  Config::stopWatch.start();
  LcdLogger::writeFileValues();

  while (!Config::stopWatch.hasReached(2000)) { }
  Config::stopWatch.stop();
  // Serial << F("Stop watch: ") << Config::stopWatch.getInterval() << endl;  // DEBUG
  pLgRunScreen1();
}
#endif


#ifdef GERMAN_TEXT
void LcdText::pLgRunScreen1() {
  LcdLogger::timer.lcdRun();
  wbLcd->home();
  //             "01234567890123456789"
  //             "> Aufzeichnung läuft"
  //             "  auf lcd00.csv     "
  wbLcd->print(F("\x01 Aufzeichnung l\xE1uft"));
  wbLcd->setCursor(10, LINE1);
  wbLcd->print(F("         "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("  auf "));
  wbLcd->print(LcdLogger::getFilename());
  printValuesStrings();
  written = true;
  erased = false;
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pLgRunScreen1() {
  LcdLogger::timer.lcdRun();
  wbLcd->home();
  //             "01234567890123456789"
  //             "> Recording to:     "
  //             "  lcd00.csv         "
  wbLcd->print(F("\x01 Recording to:     "));
  wbLcd->setCursor(10, LINE1);
  wbLcd->print(F("         "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("  "));
  wbLcd->print(LcdLogger::getFilename());
  printValuesStrings();
  written = true;
  erased = false;
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pLgRunScreen1() {
  LcdLogger::timer.lcdRun();
  textBorder();
  //          "|01234567890123456789|"
  //          "|> Aufzeichnung läuft|"
  //          "|  auf lcd00.csv     |"
  Serial << F("|> Aufzeichnung läuft|\n");
  String str = F("|  auf ");
  str += LcdLogger::getFilename();
  int len = str.length();
  while (len <= SCREEN_WIDTH) {
    str += F(" ");
    len++;
  }
  Serial << str << F("|\n");
  printValuesStrings();
  written = true;
  erased = false;
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif


#ifdef GERMAN_TEXT
void LcdText::pLgRunScreen2() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "> Für Pause: Taste  "
  //             "  kurz drücken.     "
  wbLcd->print(F("\x01 F\xF5r Pause Taste   "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("  kurz dr\xF5\x63ken.     "));
  printValuesStrings();
  written = true;
  erased = false;
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pLgRunScreen2() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "> To pause: briefly "
  //             "  press the button. "
  wbLcd->print(F("\x01 To pause: briefly "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("  press the button. "));
  printValuesStrings();
  written = true;
  erased = false;
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pLgRunScreen2() {
  textBorder();
  //          "|01234567890123456789|"
  Serial << F("|> Für Pause Taste   |\n");
  Serial << F("|  kurz drücken.     |\n");
  printValuesStrings();
  written = true;
  erased = false;
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif


#ifdef GERMAN_TEXT
void LcdText::pLgRunScreen3() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "> Für Stopp: Taste  "
  //             "  3 Sek. drücken.   "
  wbLcd->print(F("\x01 F\xF5r Stopp: Taste  "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("  3 Sek. dr\xF5\x63ken. "));
  printValuesStrings();
  written = true;
  erased = false;
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pLgRunScreen3() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "> To stop: press    "
  wbLcd->print(F("\x01 To stop: press    "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("  button for 3 sec. "));
  printValuesStrings();
  written = true;
  erased = false;
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pLgRunScreen3() {
  textBorder();
  //          "|01234567890123456789|"
  Serial << F("|> Für Stopp: Taste  |\n");
  Serial << F("|  3 Sek. drücken.   |\n");
  printValuesStrings();
  written = true;
  erased = false;
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif


#ifdef GERMAN_TEXT
void LcdText::pLgPause1() {
  LcdLogger::timer.lcdHalt();
  wbLcd->home();
  //             "01234567890123456789"
  //             "Pause. Weiter:      "
  //             "Taste kurz drücken. "
  wbLcd->print(F("Pause. Weiter:      "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("Taste kurz dr\xF5\x63ken. "));
  printValuesStrings();
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pLgPause1() {
  LcdLogger::timer.lcdHalt();
  wbLcd->home();
  //             "01234567890123456789"
  wbLcd->print(F("Pausing. Continue:  "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("briefly press button"));
  printValuesStrings();
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pLgPause1() {
  LcdLogger::timer.lcdHalt();
  textBorder();
  //          "|01234567890123456789|"
  Serial << F("|Pause. Weiter:      |\n");
  Serial << F("|Taste kurz drücken. |\n");
  printValuesStrings();
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif


#ifdef GERMAN_TEXT
void LcdText::pLgPause2() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "Pause. Für Stopp:   "
  //             "3 Sekunden drücken. "
  wbLcd->print(F("Pause. F\xF5r Stopp:   "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("3 Sek. dr\xF5\x63ken.     "));
  printValuesStrings();
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::pLgPause2() {
  wbLcd->home();
  //             "01234567890123456789"
  wbLcd->print(F("Pausing. To stop:   "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("press button 3 sec. "));
  printValuesStrings();
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif
#ifdef SERIAL_TEXT
void LcdText::pLgPause2() {
  textBorder();
  //          "|01234567890123456789|"
  Serial << F("|Pause. Für Stopp:   |\n");
  Serial << F("|3 Sekunden drücken. |\n");
  printValuesStrings();
  LcdStatesDiagram::collectOn();
  LcdFSM::resetMillis();
}
#endif


#ifndef SERIAL_TEXT
void LcdText::pLgRunBlinkOff() {
  if (!erased) {
    wbLcd->home();
    wbLcd->print(F(" "));
    written = false;
    erased = true;
  }
}
#else
void LcdText::pLgRunBlinkOff() {
  if (!erased) {
    Serial << F("Pfeil > aus\n");
    written = false;
    erased = true;
  }
}
#endif


#ifdef GERMAN_TEXT
void LcdText::p2Countdown() {
  if (!once2) {
    wbLcd->home();
    //             "01234567890123456789"
    //             "Stopp in            "
    //             "2 Sekunden.         "
    wbLcd->print(F("Stopp in            "));
    wbLcd->setCursor(0, LINE1);
    wbLcd->print(F("2 Sekunden.         "));
    once2 = true;
    once1 = once0 = false;
    LcdFSM::resetMillis();
  }
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::p2Countdown() {
  if (!once2) {
    wbLcd->home();
    //             "01234567890123456789"
    wbLcd->print(F("Stopping in         "));
    wbLcd->setCursor(0, LINE1);
    wbLcd->print(F("2 seconds.          "));
    once2 = true;
    once1 = once0 = false;
    LcdFSM::resetMillis();
  }
}
#endif
#ifdef SERIAL_TEXT
void LcdText::p2Countdown() {
  if (!once2) {
    textBorder();
    //          "|01234567890123456789|"
    Serial << F("|Stopp in 2 Sekunden |\n");
    once2 = true;
    once1 = once0 = false;
    LcdFSM::resetMillis();
  }
}
#endif


#ifdef GERMAN_TEXT
void LcdText::p1Countdown() {
  if (!once1) {
    wbLcd->home();
    //             "01234567890123456789"
    //             "Stopp in            "
    //             "1 Sekunde.          "
    wbLcd->print(F("Stopp in            "));
    wbLcd->setCursor(0, LINE1);
    wbLcd->print(F("1 Sekunde.          "));
    once1 = true;
    once2 = once0 = false;
  }
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::p1Countdown() {
  if (!once1) {
    wbLcd->home();
    //             "01234567890123456789"
    wbLcd->print(F("Stopping in         "));
    wbLcd->setCursor(0, LINE1);
    wbLcd->print(F("1 second.           "));
    once1 = true;
    once2 = once0 = false;
  }
}
#endif
#ifdef SERIAL_TEXT
void LcdText::p1Countdown() {
  if (!once1) {
    //          "|01234567890123456789|"
    textBorder();
    Serial << F("|Stopp in 1 Sekunde  |\n");
    once1 = true;
    once2 = once0 = false;
  }
}
#endif


#ifdef GERMAN_TEXT
void LcdText::p0Countdown() {
  if (!once0) {
    wbLcd->home();
    //             "01234567890123456789"
    //             "Aufzeichnung        "
    //             "gestoppt.           "
    wbLcd->print(F("Aufzeichnung        "));
    wbLcd->setCursor(0, LINE1);
    wbLcd->print(F("gestoppt.           "));
    once0 = true;
    once2 = once1 = false;

    // Disable LcdButton
    LcdButton::disable();
    LcdLogger::buttonLocked = true;
    LcdLogger::lockingTimepoint = millis();
  }
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::p0Countdown() {
  if (!once0) {
    wbLcd->home();
    //             "01234567890123456789"
    wbLcd->print(F("Recording stopped.  "));
    wbLcd->setCursor(0, LINE1);
    wbLcd->print(F("                    "));
    once0 = true;
    once2 = once1 = false;

    // Disable LcdButton
    LcdButton::disable();
    LcdLogger::buttonLocked = true;
    LcdLogger::lockingTimepoint = millis();
  }
}
#endif
#ifdef SERIAL_TEXT
void LcdText::p0Countdown() {
  if (!once0) {
    textBorder();
    //          "|01234567890123456789|"
    Serial << F("|Aufzeichnung        |\n");
    Serial << F("|gestoppt.           |\n");
    once0 = true;
    once2 = once1 = false;

    // Disable LcdButton
    LcdButton::disable();
    LcdLogger::buttonLocked = true;
    LcdLogger::lockingTimepoint = millis();
  }
}
#endif


#ifdef GERMAN_TEXT
void LcdText::printSdNotPresent() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "SD-Karte nicht er-  "
  //             "kannt: eingsteckt,  "
  //             "formatiert...?      "
  wbLcd->print(F("SD-Karte nicht er-  "));
  wbLcd->print(LcdLogger::getFilename());
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("kannt: eingsteckt,  "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("formatiert...?      "));
  printClearLastLine();
  LcdStatesDiagram::setActive(false);
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::printSdNotPresent() {
  wbLcd->home();
  //             "01234567890123456789"
  wbLcd->print(F("Could not detect    "));
  wbLcd->print(LcdLogger::getFilename());
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("SD card: present,   "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("formatted...?       "));
  printClearLastLine();
  LcdStatesDiagram::setActive(false);
}
#endif
#ifdef SERIAL_TEXT
void LcdText::printSdNotPresent() {
  textBorder();
  //          "|01234567890123456789|"
  Serial << F("|SD-Karte nicht er-  |\n");
  Serial << F("|kannt: eingsteckt,  |\n");
  Serial << F("|formatiert...?      |\n");
  printClearLastLine();
  LcdStatesDiagram::setActive(false);
}
#endif


#ifdef GERMAN_TEXT
void LcdText::printFileCount99() {
  wbLcd->home();
  //             "01234567890123456789"
  //             "Datei-Nr 99 erreicht"
  //             "Bitte Dateien auf   "
  //             "SD-Karte löschen.   "
  wbLcd->print(F("Datei-Nr 99 erreicht"));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("Bitte Dateien auf   "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("SD-Karte l\xEFschen.   "));
  printClearLastLine();
  LcdStatesDiagram::setActive(false);
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::printFileCount99() {
  wbLcd->home();
  //             "01234567890123456789"
  wbLcd->print(F("File count 99       "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("reached: Please     "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("exchange SD card.   "));
  printClearLastLine();
  LcdStatesDiagram::setActive(false);
}
#endif
#ifdef SERIAL_TEXT
void LcdText::printFileCount99() {
  textBorder();
  //           "|01234567890123456789|"
  Serial << F("|Datei-Nr 99 erreicht|\n");
  Serial << F("|Bitte Dateien auf   |\n");
  Serial << F("|SD-Karte löschen.   |\n");
  printClearLastLine();
  LcdStatesDiagram::setActive(false);
}
#endif


#ifdef GERMAN_TEXT
void LcdText::printSerialActive() {
  //             "01234567890123456789"
  //             "   Serial aktiv,    "
  //             "   Taste inaktiv.   "
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("   Serial aktiv,    "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F("   Taste inaktiv.   "));
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::printSerialActive() {
  wbLcd->setCursor(0, LINE2);
  //             "01234567890123456789"
  wbLcd->print(F("  Serial active,    "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F("  button inactive.  "));
}
#endif
#ifdef SERIAL_TEXT
void LcdText::printSerialActive() {
  //          "|01234567890123456789|"
  Serial << F("|   Serial aktiv,    |\n");
  Serial << F("|   Taste inaktiv.   |\n");
  textBorder();
}
#endif


#ifdef GERMAN_TEXT
void LcdText::printSerial1Active() {
  wbLcd->setCursor(0, LINE2);
  //             "01234567890123456789"
  wbLcd->print(F("   Serial1 aktiv,   "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F("   Taste inaktiv.   "));
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::printSerial1Active() {
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("  Serial1 active,   "));
  wbLcd->setCursor(0, LINE3);
  wbLcd->print(F("  button inactive.  "));
}
#endif
#ifdef SERIAL_TEXT
void LcdText::printSerial1Active() {
  //          "|01234567890123456789|"
  Serial << F("|   Serial1 aktiv,   |\n");
  Serial << F("|   Taste inaktiv.   |\n");
  textBorder();
}
#endif


void LcdText::wifiSymbol() {
  wbLcd->setCursor(SCREEN_WIDTH - 1, LINE0);
  wbLcd->print(F("\x02"));
}


void LcdText::wifiNoSymbol() {
  wbLcd->setCursor(SCREEN_WIDTH - 1, LINE0);
  wbLcd->print(F(" "));
}


void LcdText::powerOffSymbol() {
  wbLcd->setCursor(SCREEN_WIDTH - 1, LINE0);
  wbLcd->print(F("\x03"));
}


#ifdef GERMAN_TEXT
void LcdText::powerOffText() {
  wbLcd->clear();
  //             "01234567890123456789"
  //             "Bitte jetzt Taste   "
  //             "loslassen.          "
  //             "Auf Wiedersehen!    "
  wbLcd->print(F("Bitte Taste jetzt   "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("loslassen.          "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("Auf Wiedersehen!    "));
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::powerOffText() {
  wbLcd->clear();
  //             "01234567890123456789"
  //             "Please release      "
  //             "button now.         "
  //             "Good bye!           "
  wbLcd->print(F("Please release      "));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("button now.         "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("Good bye!           "));
}
#endif
#ifdef SERIAL_TEXT
void LcdText::powerOffText() {
  textBorder();
  //           "|01234567890123456789|"
  Serial << F("|Bitte Taste jetzt   |\n");
  Serial << F("|loslassen.          |\n");
  Serial << F("|Auf Wiedersehen!    |\n");
  printClearLastLine();
  LcdStatesDiagram::setActive(false);
}
#endif


#ifdef GERMAN_TEXT
void LcdText::error(String str) {
  wbLcd->clear();
  //             "01234567890123456789"
  //             "SD card Fehler      "
  //             "Bitte Fehler beheben"
  //             "und neu starten.    "
  wbLcd->print(str);
  wbLcd->print(F("Fehler"));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("Bitte Fehler beheben"));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("und neu starten.    "));
  // Endless loop
  while (true) { }
}
#endif
#ifdef ENGLISH_TEXT
void LcdText::error(String str) {
  wbLcd->clear();
  //             "01234567890123456789"
  //             "SD card error       "
  //             "Please eliminate    "
  //             "error and restart.  "
  wbLcd->print(str);
  wbLcd->print(F("error"));
  wbLcd->setCursor(0, LINE1);
  wbLcd->print(F("Please eliminate    "));
  wbLcd->setCursor(0, LINE2);
  wbLcd->print(F("error and restart.  "));
  // Endless loop
  while (true) { }
}
#endif
#ifdef SERIAL_TEXT
void LcdText::error(String str) {
  textBorder();
  Serial << "|" << str;
  //           "|01234567890123456789|"
  Serial << F("error|\n");
  Serial << F("|Please eliminate    |\n");
  Serial << F("|error and restart.  |\n");
  printClearLastLine();
  // Endless loop
  while (true) { }
}
#endif
