/*
 * Monitor.cpp
 *
 * Created on: 18 Feb 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "Monitor.h"
#include "Config.h"
#include "Commands.h"


Timer Monitor::timer = Timer("monitor");
bool Monitor::tempOn = false;
bool Monitor::conductivityOn = false;
bool Monitor::turbidityOn = false;


void Monitor::setTempOn(bool flag) {
  tempOn = flag;
}


bool Monitor::isTempOn() {
  return tempOn;
}


void Monitor::setConductivityOn(bool flag) {
  conductivityOn = flag;
}


bool Monitor::isConductivityOn() {
  return conductivityOn;
}


void Monitor::setTurbidityOn(bool flag) {
  turbidityOn = flag;
}


bool Monitor::isTurbidityOn() {
  return turbidityOn;
}


void Monitor::printTempSettings() {
  Config::outStream->print(F("temp monitoring on: "));
  Config::outStream->println(tempOn ? "yes" : "no");
}


void Monitor::printConductivitySettings() {
  Config::outStream->print(F("conductivity monitoring on: "));
  Config::outStream->println(conductivityOn ? "yes" : "no");
}


void Monitor::printTurbiditySettings() {
  Config::outStream->print(F("turbidity monitoring on: "));
  Config::outStream->println(turbidityOn ? "yes" : "no");
}


void Monitor::handleMonitor() {
  if (timer.isRunning()) {
    if (Monitor::timer.isWithinMeasuringPeriod()) {
      Commands::printCurrentTimeLn();
      // Print values of selected sensors to monitor
      Commands::printMonitorTemp();
      Commands::printMonitorConductivity();
      Commands::printMonitorTurbidity();
    }
  }
}


Monitor::Monitor() { }
