/*
 * Temperature.cpp
 *
 * Created on: 18 Feb 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "Temperature.h"
#include <Arduino.h>
#include "Config.h"
#include "TwoPointCalibrationHelper.h"
#include "Timeout.h"

#define TIMEOUT_PERIOD_IN_SECS 3


TwoPointCalibrationHelper Temperature::helper;

char Temperature::buf[20];

OneWire Temperature::oneWire = OneWire(Config::TEMP_PIN);

DallasTemperature Temperature::sensor = DallasTemperature(&oneWire);

float Temperature::lastValue = 0;

Timeout Temperature::timeout = Timeout(0, TIMEOUT_PERIOD_IN_SECS);


void Temperature::begin() {
  // Read values from preferences.
  // At class loading time preferences are not yet initialised,
  // therefore cannot use c'tor.
  helper.init(
    TEMPERATURE_STR, TEMP_RAW_LOW_STR, TEMP_RAW_HIGH_STR,
    TEMP_IDEAL_LOW_STR, TEMP_IDEAL_HIGH_STR, TEMP_LAST_DATE_STR,
    readRaw, formatString);
  sensor.begin();
  timeout.start();
  sensor.requestTemperatures();
  // Read first temperature value
  // More than one IC on the same bus possible.
  // If only one: 0 refers to the first IC on the wire
  lastValue = sensor.getTempCByIndex(0);;
}


float Temperature::readRaw() {
  // Only read a value with a time period
  if (timeout.hasTimedout()) {
    timeout.start();
    sensor.requestTemperatures();
    lastValue = sensor.getTempCByIndex(0);
  }
  return lastValue;
}


String Temperature::formatString(float temp, bool withUnit) {
  // Print number in the format:  1.1  -0.3  6.8  33.4
  // Workaround, because...
  // sprintf(buf, "  %.1f°C", read());  // ... does NOT work in Arduino
  int tempTimesTen = (int)(10 * temp);
  bool minus = false;
  if (tempTimesTen < 0) {
    tempTimesTen *= -1;
    minus = true;
  }
  int prePoint = tempTimesTen / 10;
  int pastPoint = tempTimesTen % 10;
  sprintf(
    buf, "%s%s%i.%i%s",
    temp >= 1.0 ? "" : " ",
    minus ? "-" : "",
    prePoint, pastPoint,
    // withUnit ? " °C" : ""                        // Windows Excel
    // withUnit ? " \xB0" "C" : ""   // \xB0 = '°'  // Ubuntu/iOS/Android Excel
    withUnit ? " degC" : ""
  );
  return String(buf);
}
