/*
 * WbLCD.cpp
 *
 * Created on: 22 May 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "WbLCD.h"
#include "Streaming.h"
#include "Config.h"
#include "LcdText.h"
#include "WbPreferences.h"


#define NUM_LINES 4
#define NUM_CHARS 20
#define LCD_I2C_ADDRESS 0x27
// Addresses of LCD display symbol table
#define ARROW_ADDR 1
#define WIFI_SYMBOL_ADDR 2
#define POWEROFF_SYMBOL_ADDR 3

const byte WbLCD::ARROW[8] = {
  B10000,
  B11000,
  B11100,
  B11110,
  B11100,
  B11000,
  B10000,
  B00000,
};

const byte WbLCD::WIFI_SYMBOL[8] = {
  B01110,
  B10001,
  B10101,
  B10001,
  B01110,
  B00100,
  B00100,
  B00100,
};

const byte WbLCD::POWEROFF_SYMBOL[8] = {
  B00000,
  B00100,
  B10101,
  B10101,
  B10101,
  B10001,
  B01110,
  B00000,
};


LiquidCrystal_I2C* WbLCD::lcd = nullptr;

Timeout WbLCD::timer(1);  // Default 1 minute timeout


void WbLCD::begin() {
  lcd = new LiquidCrystal_I2C(LCD_I2C_ADDRESS, NUM_CHARS, NUM_LINES);
  lcd->init();
  lcd->clear();

  lcd->createChar(ARROW_ADDR, (uint8_t*)ARROW);
  lcd->createChar(WIFI_SYMBOL_ADDR, (uint8_t*)WIFI_SYMBOL);
  lcd->createChar(POWEROFF_SYMBOL_ADDR, (uint8_t*)POWEROFF_SYMBOL);

  lcd->backlight();
  timer.enable();

  pinMode(Config::TOUCH_BUTTON_PIN, INPUT);

  Serial << (F("LCD display initialised\n"));
  LcdText::powerOnScreen();
}


LiquidCrystal_I2C* WbLCD::getLcd() {
  return lcd;
}


void WbLCD::outLcdPrintError(String str) {
  lcd->clear();
  lcd->print(str);
  lcd->print(F(": not inserted?"));
  lcd->setCursor(0, 1);
  lcd->print(F("fix error and restart system!"));
  while (true)   { }
}


void WbLCD::backlightOn() {
  lcd->backlight();
  int period = WbPreferences::getBacklightTimeout();
  if (period == 0) {
    // Set backlight on for at least one minute
    period = 1;
  }
  timer.setPeriod(period);
  timer.start();
}


void WbLCD::backlightOff() {
  // Used for Devel: "LCD backlight on/off", first priority
  if (!timer.isEnabled())   return;

  // Ignore backlight off due to preferences
  if (WbPreferences::getBacklightTimeout() < 0) {
    return;
  }
  lcd->noBacklight();
}


void WbLCD::backlightCheck() {
  if (timer.hasTimedout()) {
    backlightOff();
  }
  // Has button been touched?
  if (digitalRead(Config::TOUCH_BUTTON_PIN) == HIGH) {
    backlightOn();
  }
}
