/*
 * WbRTC.cpp
 *
 * Created on: 7 Feb 2021
 * Author: ruedi.mueller@fhnw.ch
 */

#include "WbRTC.h"
#include <Arduino.h>
#include <Wire.h>
#include <sys/time.h>
#include "Streaming.h"
#include "Config.h"


RTC_PCF8523 WbRTC::rtc;
DateTime WbRTC::savedCurrentDateTime;
char WbRTC::buf[20];


void WbRTC::updateSystemTime() {
  saveCurrentDateTime();
  // Init system time, important for file timestamps
  int32_t unixtime = savedCurrentDateTime.unixtime();
  suseconds_t millis = 0;
  timeval epoch = { unixtime, millis };
  const timezone* tz = 0;
  settimeofday((const timeval*)&epoch, tz);
}


bool WbRTC::begin() {
  Wire.begin();
  Wire.setClock(800000);   // Maximum
  Serial << F("Wire: ") << Wire.getClock() << endl;

  Wire.beginTransmission(PCF8523_ADDRESS);   // adalogger RTC I2C address
  if (Wire.endTransmission() == 0) {
	  rtc.start();
    updateSystemTime();
    return true;
  }
  return false;
}


void WbRTC::start() {
  rtc.start();
}


void WbRTC::stop() {
  rtc.stop();
}


unsigned int WbRTC::savedYear() {
  return savedCurrentDateTime.year();
}
  
unsigned char WbRTC::savedMonth() {
  return savedCurrentDateTime.month();
}

unsigned char WbRTC::savedDay() {
  return savedCurrentDateTime.day();
}

unsigned char WbRTC::savedHour() { 
  return savedCurrentDateTime.hour();
}

unsigned char WbRTC::savedMinute() {
  return savedCurrentDateTime.minute();
}

unsigned char WbRTC::savedSecond() {
  return savedCurrentDateTime.second();
}

unsigned char WbRTC::savedDayOfTheWeek() {
  int idx = savedCurrentDateTime.dayOfTheWeek() - 1;
  if (idx < 0)   idx = 6;
  return idx;
}

void WbRTC::saveCurrentDateTime() {
  savedCurrentDateTime = rtc.now();
}


void WbRTC::setTime(int hh, int mm, int ss) {
  DateTime now = rtc.now();
  rtc.adjust(DateTime(now.year(), now.month(), now.day(), hh, mm, ss));
  updateSystemTime();
}


void WbRTC::setDate(int dd, int mm, int yyyy) {
  DateTime now = rtc.now();
  rtc.adjust(DateTime(yyyy, mm, dd, now.hour(), now.minute(), now.second()));
  updateSystemTime();
}


void WbRTC::setWithPreprocessorDateTime() {
  // Get date and time from host OS while compiling
  DateTime now(DateTime(F(__DATE__), F(__TIME__)));
  // Add 40 seconds: estimated interval till RTC will be initialised with date and time 
  DateTime future(now + TimeSpan(0, 0, 0, 40));
  rtc.adjust(future);
  updateSystemTime();
}


DateTime WbRTC::getSavedCurrentDateTime() {
  return savedCurrentDateTime;
}


String WbRTC::getSavedDateString() {
  sprintf(
    buf, "%02d.%02d.%4d",
    savedDay(), savedMonth(), savedYear()
  );
  return String(buf);
}


String WbRTC::getSavedTimeString() {
  sprintf(
    buf, "%02d:%02d:%02d",
    savedHour(), savedMinute(), savedSecond()
  );
  return String(buf);
}


// This is *not* RTC time but ESP32 time (used when files are written)
void WbRTC::printSystemTime() {
  char buffer[64];
  struct timeval tv;
  gettimeofday(&tv, NULL);
  
  time_t t = tv.tv_sec;
  struct tm* info = localtime(&t);
  
  strftime(buffer, sizeof buffer, "System date and time: %A, %d %B %Y  ", info);
  Config::outStream->printf("%s", buffer);
  strftime(buffer, sizeof buffer, "%H:%M:%S\n", info);
  Config::outStream->printf("%s", buffer);
}
